/*
 * $Id: ActivityEdit.cpp,v 1.1 2002/10/31 20:06:18 robert Exp $
 *
 * Copyright (C) 2002 Dafydd Walters <dwalters@users.sourceforge.net>
 * 
 * This program may be modified or redistributed under the terms of the GNU
 * General Public License (GPL), version 2 or later. A copy of the GNU GPL 
 * should have accompanied the source code of this program. If not, please 
 * write to the Free Software Foundation Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * This program is distributed without any warranty.
 */ 

#include "ActivityEdit.h"
#include "TimeSleuthDocument.h"
#include "CategoryEdit.h"

#include <qpe/resource.h>
#include <qlist.h>
#include <qmessagebox.h>
#include <qcombobox.h>
#include <qtoolbutton.h>
#include <qlineedit.h>
#include <qapplication.h>

ActivityEdit::ActivityEdit(Activity* a, TimeSleuthDocument* doc, 
    QWidget* parent, const char* name, bool modal, WFlags fl) :
    ActivityEditBase(parent, name, modal, fl), currentActivity(a), 
    category(0), document(doc), activeList(0)
{
    setupActivity(); 
    setupCategories();

    connect(comboCategory, SIGNAL(activated(int)), this, 
            SLOT(categoryActivated(int)));
    connect(btnCategory, SIGNAL(clicked()), this, SLOT(maintainCategories()));
}

ActivityEdit::~ActivityEdit()
{
    if (activeList)
    {
        activeList->clear();
	delete activeList;
    }
}

void ActivityEdit::setupActivity()
{
    if (currentActivity)
    {
        txtName->setText(currentActivity->name);
        category = currentActivity->category;
    }
}

void ActivityEdit::categoryActivated(int index)
{
    category = activeList->at(index);
}

void ActivityEdit::setupCategories()
{
    if (document->configuration.mode() == Configuration::GROUP_MODE)
        btnCategory->hide();
    
    if(!activeList)
        activeList = new QList<ActivityCategory>;

    activeList->clear();
    comboCategory->clear();
    
    QListIterator<ActivityCategory> li(document->activityCategories);

    bool found = false;
    for( int i = 0; li.current(); ++li)
    {
        ActivityCategory* c = li.current();
        if ((!c->deleted) || (category == c))
	{
            // Only include categories that have not been deleted, 
            // or one that has been deleted but this activity is still 
            // associated with it (cannot choose a deleted category 
            // for a New activity, though).
            activeList->append(c);
            if (c->deleted)
                comboCategory->insertItem(Resource::loadPixmap( "trash" ),
                                          c->name, i);
            else
                comboCategory->insertItem(c->name, i);
            if (category == c)
            {
                found = true;
                comboCategory->setCurrentItem(i);
            }

            ++i;
        }
    }

    if (!found)
    {
        category = activeList->first();
        comboCategory->setCurrentItem(0);
    }
}

void ActivityEdit::accept()
{
    QString name = txtName->text().stripWhiteSpace();
    
    if ((name == "--sync--") || (name == "--start--") || 
        (name == "--end--"))
    {
        QMessageBox::warning(this, 
            tr("Activity Error"),
            tr("This name is not valid.\n"
               "Please select another."),
            QMessageBox::Ok, 0);
        return;
    }
    
    if (category == 0)
    {
        QMessageBox::warning(this, tr("Activity Error"), 
            tr("Activity must have a category"), QMessageBox::Ok, 0);
        return;
    }

    if (name.isEmpty())
    {
        QMessageBox::warning(this, tr("Activity Error"), 
            tr("Activity must have a name"), QMessageBox::Ok, 0);
        return;
    }

    QListIterator<Activity> li(document->activities);
    for( ; li.current(); ++li)
    {
        Activity* a = li.current();
        if ((a != currentActivity) && 
            (a->name == name) &&
            (!a->deleted))
        {
            QMessageBox::warning(this, tr("Activity Error"), 
                tr("Activity name must be unique"), QMessageBox::Ok, 0);
            return;
        }
    }

    ActivityEditBase::accept();
}

void ActivityEdit::maintainCategories()
{
    CategoryEdit dlg(0, document, this, 0, true);
    dlg.setCaption(tr("TimeSleuth - New Category"));
    dlg.showMaximized();
    if (dlg.exec())
    {
        ActivityCategory* c = new ActivityCategory(document->allocateId(), 
            dlg.txtName->text().stripWhiteSpace());
        document->activityCategories.append(c);
        category = c;
        document->activityCategories.sort();
        setupCategories();
    }
}
