/*
 * $Id: ActivityList.cpp,v 1.2 2003/01/25 16:20:06 robert Exp $
 *
 * Copyright (C) 2002 Dafydd Walters <dwalters@users.sourceforge.net>
 * 
 * This program may be modified or redistributed under the terms of the GNU
 * General Public License (GPL), version 2 or later. A copy of the GNU GPL 
 * should have accompanied the source code of this program. If not, please 
 * write to the Free Software Foundation Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * This program is distributed without any warranty.
 */ 

#include "ActivityList.h"
#include "TimeSleuthDocument.h"
#include "Activity.h"
#include "ActivityEdit.h"
#include "TSTable.h"

#include <qapplication.h>
#include <qlist.h>
#include <qtoolbutton.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <qaccel.h>
#include <qpe/qpemessagebox.h>

ActivityList::ActivityList(TimeSleuthDocument* doc, QWidget* parent, 
    const char* name, WFlags fl) : ActivityListBase(parent, name, fl),
    document(doc), selectedActivity(0), activeList(0)
{
    setupList();
    updateView();

    accel = new QAccel(this, "accel");
    accel->insertItem(Qt::Key_Escape, 200);
    accel->connectItem(200, this, SLOT(close()));

    connect(tblActivityList, SIGNAL(currentChanged(int,int)), this,
            SLOT(cellChanged(int,int)));
    connect(btnNew, SIGNAL(clicked()), this, SLOT(newActivity()));
    connect(btnEdit, SIGNAL(clicked()), this, SLOT(editActivity()));
    connect(btnDelete, SIGNAL(clicked()), this, SLOT(deleteActivity()));
    connect(tblActivityList, SIGNAL( ok() ), this, SLOT( editActivity() ) );
}

ActivityList::~ActivityList()
{
    if (activeList)
    {
        activeList->clear();
        delete activeList;
    }
}

void ActivityList::editActivity()
{
    ActivityEdit dlg(selectedActivity, document, this, 0, true);
    dlg.setCaption(tr("TimeSleuth - Edit Activity"));
    dlg.showMaximized();
    if (dlg.exec())
    {
        dlg.currentActivity->name = dlg.txtName->text().stripWhiteSpace();
        dlg.currentActivity->category = dlg.category;
        document->activities.sort();
        updateView();
    }

}

void ActivityList::deleteActivity()
{
    Activity* a = selectedActivity;

    if (!a)
        return;

    int numTasks = 0;
    QListIterator<TSTask> li(document->tasks);
    for( ; li.current(); ++li)
    {
        TSTask* t = li.current();
        if (t->activity == a)
        {
            numTasks++;
        }
    }

    if (numTasks > 0)
    {
        if (QMessageBox::warning(this, tr("Delete Activity"), tr("There are "
            "%1 task(s) using\n"
            "this activity. Are you sure\n"
            "you want to delete\n"
            "%2?"
            ).arg(numTasks).arg(a->name), QMessageBox::Yes, 
            QMessageBox::No|QMessageBox::Default |
            QMessageBox::Escape) != QMessageBox::Yes)
        {
            return;
        }
    }
    else if (QMessageBox::warning(this, tr("Delete Activity"),
             tr("Are you sure you want to delete\n%1?").arg(a->name),
             QMessageBox::Yes, QMessageBox::No | QMessageBox::Default |
             QMessageBox::Escape) != QMessageBox::Yes)
    {
        return;
    }

    // Remove activity
    a->deleted = true;

    // Make sure there's no shortcut button for this activity
    for( li.toFirst(); li.current(); ++li)
    {
        TSTask* t = li.current();
        if (t->activity == a)
        {
            t->shortcutPos = 0;
        }
    }
    // Update the list display
    selectedActivity = 0;
    updateView();
}

void ActivityList::newActivity()
{
    ActivityEdit dlg(0, document, this, 0, true);
    dlg.setCaption(tr("TimeSleuth - New Activity"));
    dlg.showMaximized();
    if (dlg.exec())
    {
        Activity* a = new Activity(document->allocateId(), 
            dlg.txtName->text().stripWhiteSpace(), dlg.category);
        document->activities.append(a);
        selectedActivity = a;
        document->activities.sort();
        updateView();
    }
}

void ActivityList::hide()
{
    ActivityListBase::hide();
    qApp->exit_loop();
}

void ActivityList::updateView()
{
    if (!activeList)
        activeList = new QList<Activity>;

    activeList->clear();
    
    int col = tblActivityList->currentColumn();
    if (col < 0)
        col = 0;

    tblActivityList->blockSignals(true);
    tblActivityList->setPaintingEnabled( false );
//    tblActivityList->setUpdatesEnabled( false );
//    tblActivityList->viewport()->setUpdatesEnabled(false);

    tblActivityList->clear();

    int row = 0;
    tblActivityList->setNumRows(document->getNumActiveActivities());
    QListIterator<Activity> li(document->activities);
    for( ; li.current(); ++li)
    {
        Activity* a = li.current();

        if (!a->deleted)
        {
            activeList->append(a);

            QTableItem *ti;

            // Activity name
            ti = new QTableItem(tblActivityList, QTableItem::Never, a->name);
            tblActivityList->setItem(row, 0, ti);

            // Category name
            ti = new QTableItem(tblActivityList, QTableItem::Never, 
                                a->category->name);
            tblActivityList->setItem(row, 1, ti);

            ++row;
        }
    }
    tblActivityList->setColumnWidth(0, 150);
    tblActivityList->setColumnWidth(1, 76);

    int activityRow = activeList->findRef(selectedActivity);
    if (activityRow != -1)
    {
        tblActivityList->ensureCellVisible(activityRow, col);
        tblActivityList->setCurrentCell(activityRow, col);
    }
    else if (row > 0)
    {
        tblActivityList->ensureCellVisible(tblActivityList->numRows() - 1, col);
        tblActivityList->setCurrentCell(tblActivityList->numRows() - 1, col);
        selectedActivity = activeList->at(
                           tblActivityList->numRows() - 1);
    }
    else
    {
        selectedActivity = 0;
    }

    // Update the enabled/disabled state of buttons
    updateEnabled();

//    tblActivityList->setUpdatesEnabled( true );
//    tblActivityList->viewport()->setUpdatesEnabled(true);
    tblActivityList->setPaintingEnabled(true);
    tblActivityList->blockSignals(false);
}

void ActivityList::cellChanged(int row, int col)
{
    if (row < 0 || col < 0)
    {
        selectedActivity = 0;
        updateEnabled();
        return;
    }

    selectedActivity = activeList->at(row);
    updateEnabled();
}

void ActivityList::updateEnabled()
{
    if (document->getNumActiveCategories() == 0)
        btnNew->setEnabled(false);
    else
        btnNew->setEnabled(true);

    if (selectedActivity)
    {
        btnEdit->setEnabled(true);
        btnDelete->setEnabled(true);
    }
    else
    {
        btnEdit->setEnabled(false);
        btnDelete->setEnabled(false);
    }
}

void ActivityList::setupList()
{
    tblActivityList = new TSTable( this, "tblActivityList" );
    tblActivityList->setGeometry( QRect( 5, 5, 230, 250 ) ); 
    tblActivityList->setNumRows( 0 );
    tblActivityList->setNumCols( 0 );

    tblActivityList->blockSignals(true);

    // Set the selection mode
    tblActivityList->setSelectionMode(QTable::NoSelection);

    // Hide left header of task list
    tblActivityList->setLeftMargin(0);
    tblActivityList->verticalHeader()->hide();

    // Set number of table columns
    tblActivityList->setNumCols(2);

    // Set top header of task list
    QHeader* pHeader = tblActivityList->horizontalHeader();
    pHeader->setLabel(0, tr("Activity"));
    pHeader->setLabel(1, tr("Category"));

    tblActivityList->blockSignals(false);
}


