/*
 * $Id: CategoryList.cpp,v 1.2 2003/01/25 16:20:06 robert Exp $
 *
 * Copyright (C) 2002 Dafydd Walters <dwalters@users.sourceforge.net>
 * 
 * This program may be modified or redistributed under the terms of the GNU
 * General Public License (GPL), version 2 or later. A copy of the GNU GPL 
 * should have accompanied the source code of this program. If not, please 
 * write to the Free Software Foundation Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * This program is distributed without any warranty.
 */ 

#include "CategoryList.h"
#include "TimeSleuthDocument.h"
#include "ActivityCategory.h"
#include "CategoryEdit.h"
#include "TSTable.h"

#include <qapplication.h>
#include <qtable.h>
#include <qlist.h>
#include <qtoolbutton.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <qaccel.h>
#include <qpe/qpemessagebox.h>

CategoryList::CategoryList(TimeSleuthDocument* doc, QWidget* parent, 
    const char* name, WFlags fl) : CategoryListBase(parent, name, fl),
    document(doc), selectedCategory(0), activeList(0)
{
    setupList();
    updateView();

    accel = new QAccel(this, "accel");
    accel->insertItem(Qt::Key_Escape, 200);
    accel->connectItem(200, this, SLOT(close()));

    connect(tblCategoryList, SIGNAL(currentChanged(int,int)), this,
            SLOT(cellChanged(int,int)));
    connect(btnNew, SIGNAL(clicked()), this, SLOT(newCategory()));
    connect(btnEdit, SIGNAL(clicked()), this, SLOT(editCategory()));
    connect(btnDelete, SIGNAL(clicked()), this, SLOT(deleteCategory()));
    connect(tblCategoryList, SIGNAL( ok() ), this, SLOT( editCategory() ) );
}

CategoryList::~CategoryList()
{
    if (activeList)
    {
        activeList->clear();
	delete activeList;
    }
}

void CategoryList::editCategory()
{
    CategoryEdit dlg(selectedCategory, document, this, 0, true);
    dlg.setCaption(tr("TimeSleuth - Edit Category"));
    dlg.showMaximized();
    if (dlg.exec())
    {
        dlg.category->name = dlg.txtName->text().stripWhiteSpace();
        document->activityCategories.sort();	
        updateView();
    }
}

void CategoryList::deleteCategory()
{
    ActivityCategory* c = selectedCategory;

    if (!c)
        return;

    QList<TSTask> deletedTasks;
    QList<Activity> deletedActivities;
    QListIterator<TSTask> tli(document->tasks);
    QListIterator<Activity> ali(document->activities);

    for( ; ali.current(); ++ali)
    {
        Activity* a = ali.current();
        if (a->category == c)
        {
            deletedActivities.append(a);
            for(tli.toFirst(); tli.current(); ++tli)
            {
                TSTask* t = tli.current();
                if (t->activity == a)
                {
                    deletedTasks.append(t);
                }
            }
        }
    }

    if (deletedTasks.count() > 0)
    {
        if (QMessageBox::warning(this, tr("Delete Category"), tr("There are "
            "%1 task(s) and %2\n"
            "activities using this category.\n"
            "Deleting the category will\n"
            "automatically delete all the\n"
            "associated activities.\n\n"
            "Proceed?").arg(deletedTasks.count()).arg(deletedActivities.
            count()),
            QMessageBox::Yes, 
            QMessageBox::No|QMessageBox::Default|
            QMessageBox::Escape) != QMessageBox::Yes)
        {
            return;
        }
    }
    else if (deletedActivities.count() > 0)
    {
        if (QMessageBox::warning(this, tr("Delete Category"), tr("There are "
            "%1 activities(s) using\n"
            "this category. Deleting the\n"
            "category will automatically delete\n"
            "all the associated activities(s).\n\n"
            "Proceed?").arg(deletedActivities.count()), QMessageBox::Yes, 
            QMessageBox::No|QMessageBox::Default|
            QMessageBox::Escape) != QMessageBox::Yes)
        {
            return;
        }
    }
    else if (QMessageBox::warning(this, tr("Delete Category"),
             tr("Are you sure you want to delete\n%1?").arg(c->name),
             QMessageBox::Yes, QMessageBox::No | QMessageBox::Default |
             QMessageBox::Escape) != QMessageBox::Yes)
    {
        return;
    }

    // Remove shortcuts for affected tasks
    TSTask* t = deletedTasks.first();
    while(t)
    {
        t->shortcutPos = 0; 
        t = deletedTasks.next();
    }

    // Remove activities
    Activity* a = deletedActivities.first();
    while(a)
    { 
        a->deleted = true;
        a = deletedActivities.next();
    }

    // Remove cateogry
    c->deleted = true;

    // Update the list display
    selectedCategory = 0;
    updateView();
}

void CategoryList::newCategory()
{
    CategoryEdit dlg(0, document, this, 0, true);
    dlg.setCaption(tr("TimeSleuth - New Category"));
    dlg.showMaximized();
    if (dlg.exec())
    {
        ActivityCategory* c = new ActivityCategory(document->allocateId(), 
            dlg.txtName->text().stripWhiteSpace());
        document->activityCategories.append(c);
        selectedCategory = c;
        document->activityCategories.sort();
        updateView();
    }
}

void CategoryList::hide()
{
    CategoryListBase::hide();
    qApp->exit_loop();
}

void CategoryList::updateView()
{
    if (!activeList)
        activeList = new QList<ActivityCategory>;

    activeList->clear();
    
    tblCategoryList->blockSignals(true);
    tblCategoryList->setPaintingEnabled( false );
//    tblCategoryList->setUpdatesEnabled( false );
//    tblCategoryList->viewport()->setUpdatesEnabled(false);

    tblCategoryList->clear();

    tblCategoryList->setNumRows(document->getNumActiveCategories());
    int row = 0;    
    QListIterator<ActivityCategory> li(document->activityCategories);
    for( ; li.current(); ++li)
    {
        ActivityCategory* c = li.current();

        if (!c->deleted)
	{
            activeList->append(c);
            QTableItem *ti;

            // Category name
            ti = new QTableItem(tblCategoryList, QTableItem::Never, c->name);
            tblCategoryList->setItem(row, 0, ti);
	    
	    ++row;
	}
    }
    tblCategoryList->setColumnWidth(0, 226);

    int categoryRow = activeList->findRef(selectedCategory);
    if (categoryRow != -1)
    {
        tblCategoryList->ensureCellVisible(categoryRow, 0);
        tblCategoryList->setCurrentCell(categoryRow, 0);
    }
    else if (row > 0)
    {
        tblCategoryList->ensureCellVisible(tblCategoryList->numRows() - 1, 0);
        tblCategoryList->setCurrentCell(tblCategoryList->numRows() - 1, 0);
        selectedCategory = activeList->at(
                           tblCategoryList->numRows() - 1);
    }
    else
    {
        selectedCategory = 0;
    }

    // Update the enabled/disabled state of buttons    
    updateEnabled();

//    tblCategoryList->setUpdatesEnabled( true );
//    tblCategoryList->viewport()->setUpdatesEnabled(true);
    tblCategoryList->setPaintingEnabled(true);
    tblCategoryList->blockSignals(false);
}

void CategoryList::cellChanged(int row, int col)
{
    if (row < 0 || col < 0)
    {
        selectedCategory = 0;
        updateEnabled();
        return;
    }

    selectedCategory = activeList->at(row);
    updateEnabled();
}

void CategoryList::updateEnabled()
{
    if (selectedCategory)
    {
        btnEdit->setEnabled(true);
        btnDelete->setEnabled(true);
    }
    else
    {
        btnEdit->setEnabled(false);
        btnDelete->setEnabled(false);
    }
}

void CategoryList::setupList()
{
    tblCategoryList = new TSTable( this, "tblCategoryList" );
    tblCategoryList->setGeometry( QRect( 5, 5, 230, 250 ) ); 
    tblCategoryList->setNumRows( 0 );
    tblCategoryList->setNumCols( 0 );

    tblCategoryList->blockSignals(true);

    // Set the selection mode
    tblCategoryList->setSelectionMode(QTable::NoSelection);

    // Hide left header of task list
    tblCategoryList->setLeftMargin(0);
    tblCategoryList->verticalHeader()->hide();

    // Set number of table columns
    tblCategoryList->setNumCols(1);

    // Set top header of task list
    QHeader* pHeader = tblCategoryList->horizontalHeader();
    pHeader->setLabel(0, tr("Category"));

    tblCategoryList->blockSignals(false);
}


