/*
 * $Id: MainWidget.cpp,v 1.1 2002/10/31 20:06:18 robert Exp $
 *
 * Copyright (C) 2002 Dafydd Walters <dwalters@users.sourceforge.net>
 * 
 * This program may be modified or redistributed under the terms of the GNU
 * General Public License (GPL), version 2 or later. A copy of the GNU GPL 
 * should have accompanied the source code of this program. If not, please 
 * write to the Free Software Foundation Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * This program is distributed without any warranty.
 */ 

#include "MainWidget.h"
#include "TimeSleuthDocument.h"
#include "Task.h"
#include "Activity.h"
#include "ActivityCategory.h"
#include "TSTable.h"

#include <qpushbutton.h>
#include <qheader.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qtoolbutton.h>
#include <qlabel.h>

#define SECONDS_PER_MINUTE 60
#define SECONDS_PER_HOUR (60*60)
#define SECONDS_PER_DAY (60*60*24)

MainWidget::MainWidget( QWidget* parent,  const char* name, WFlags fl )
    : MainWidgetBase( parent, name, fl ), selectedTask(0),
      button1Task(0), button2Task(0), button3Task(0)
{
    // Initialize the task list table
    initTable();

    // Connect signals
    connect(tblTaskList, SIGNAL(currentChanged(int, int)), 
            this, SLOT(tableCellChanged(int, int)));
}

MainWidget::~MainWidget()
{
}

void MainWidget::tableCellChanged(int row, int col)
{
    if (row < 0 || col < 0)
    {
        qDebug("row=%d col=%d", row, col);
        selectedTask = 0;
        return;
    }

    selectedTask = document()->tasks.at(row);

    emit taskChanged();
}

void MainWidget::update()
{
    int col = tblTaskList->currentColumn();

    tblTaskList->blockSignals(true);
    tblTaskList->setPaintingEnabled( false );
    tblTaskList->setUpdatesEnabled( false );
    tblTaskList->viewport()->setUpdatesEnabled(false);

    btnTask1->blockSignals(true);
    btnTask2->blockSignals(true);
    btnTask3->blockSignals(true);
    button1Task = 0;
    button2Task = 0;
    button3Task = 0;

    tblTaskList->clear();

    // Update task table and shortcut buttons
    tblTaskList->setNumRows(document()->tasks.count());
    int row = 0;
    TSTask* task = document()->tasks.first();
    while(task)
    {
        populateTableRow(task, row);
        task = document()->tasks.next();
        row++;
    }
    tblTaskList->setColumnWidth(0, 100);
    tblTaskList->setColumnWidth(1, 100);
    tblTaskList->setColumnWidth(2, 200);

    setSelectedTask(col);

    blankUnusedButtons();
    
    btnTask1->blockSignals(false);
    btnTask2->blockSignals(false);
    btnTask3->blockSignals(false);
    tblTaskList->setUpdatesEnabled( true );
    tblTaskList->viewport()->setUpdatesEnabled(true);
    tblTaskList->setPaintingEnabled(true);
    tblTaskList->blockSignals(false);
    emit taskChanged();
}

void MainWidget::setSelectedTask(int col)
{
    if (col == -1)
          col = tblTaskList->currentColumn();

    if (col < 0)
        col = 0;

    int taskRow = document()->tasks.findRef(selectedTask);
    if (taskRow != -1)
    {
        tblTaskList->ensureCellVisible(taskRow, col);
        tblTaskList->setCurrentCell(taskRow, col);
    }
    else if (tblTaskList->numRows() > 0)
    {
        tblTaskList->ensureCellVisible(tblTaskList->numRows() - 1, col);
        tblTaskList->setCurrentCell(tblTaskList->numRows() - 1, col);
        selectedTask = document()->tasks.at(tblTaskList->numRows() - 1);
    }
    else
    {
        selectedTask = 0;
    }
}

void MainWidget::updateRow(TSTask* t)
{
    int taskRow = document()->tasks.findRef(t);
    if (taskRow == -1)
    {
        // could not find row
        update();
        return;
    }

    tblTaskList->clearCell(taskRow, 0);
    tblTaskList->clearCell(taskRow, 1); 
    tblTaskList->clearCell(taskRow, 2); 

    populateTableRow(t, taskRow);
}

void MainWidget::blankUnusedButtons()
{
    if (!button1Task)
    {
        btnTask1->setText("");
        btnTask1->setEnabled(false);
        btnTask1->setOn(false);
    }
    
    if (!button2Task)
    {
        btnTask2->setText("");
        btnTask2->setEnabled(false);
        btnTask2->setOn(false);
    }
    
    if (!button3Task)
    {
        btnTask3->setText("");
        btnTask3->setEnabled(false);
        btnTask3->setOn(false);
    }
}

void MainWidget::populateTableRow(TSTask* task, int row)
{
    QTableItem *ti;

    // Start time
    ti = new QTableItem(tblTaskList, QTableItem::Never, task->startString);
    tblTaskList->setItem(row, 0, ti);

    // Stop time
    ti = new QTableItem(tblTaskList, QTableItem::Never, task->stopString);
    tblTaskList->setItem(row, 1, ti);

    // Activity description
    ti = new QTableItem(tblTaskList, QTableItem::Never, task->activity->name);
    tblTaskList->setItem(row, 2, ti);
    
    switch (task->shortcutPos)
    {
        case 1 :
            setButton(btnTask1, &button1Task, task);
            break;
        case 2 :
            setButton(btnTask2, &button2Task, task);
            break;
        case 3 :
            setButton(btnTask3, &button3Task, task);
            break;
    }
}

void MainWidget::setButton(QToolButton* btnTask, TSTask** buttonTask, 
                           TSTask* task)
{
    if (btnTask->text() != task->activity->name)
        btnTask->setText(task->activity->name);

    if (!btnTask->isEnabled())
        btnTask->setEnabled(true);

    if (btnTask->isOn() == task->stop().isValid())
        btnTask->setOn(!task->stop().isValid());

    *buttonTask = task;
}

void MainWidget::updateShortcutTimers()
{
    QDateTime dt = ((TimeSleuthMainWindow*)parent())->getTSTime(true);

    updateShortcutTimer(button1Task, dt, lblTaskStatus1);
    updateShortcutTimer(button2Task, dt, lblTaskStatus2);
    updateShortcutTimer(button3Task, dt, lblTaskStatus3);
}

void MainWidget::updateShortcutTimer(TSTask* buttonTask, QDateTime dt, 
                                     QLabel* lblTaskStatus)
{
    if (buttonTask)
    {
        if (buttonTask->stop().isValid())
            lblTaskStatus->setText("");
        else
        {
            int secs = buttonTask->start().secsTo(dt);
            if (secs < 0)
                lblTaskStatus->setText("");  // task is in the future
            else
            {
                QString s;    
                char sep[2] = {0,0};
                sep[0] = (secs % 2) ? ':' : ' ';
                int days = secs / SECONDS_PER_DAY;
                secs -= (days * SECONDS_PER_DAY);
                int hours = secs / SECONDS_PER_HOUR;
                secs -= (hours * SECONDS_PER_HOUR);
                int mins = secs / SECONDS_PER_MINUTE;
                if (days > 0)
                    s.sprintf("%dd  %d%s%02d", days, hours, sep, mins);
                else   
                    s.sprintf("%d%s%02d", hours, sep, mins);
                lblTaskStatus->setText(s);
            }
        }
    }
    else
        lblTaskStatus->setText("");
}

TimeSleuthDocument* MainWidget::document() 
{
    return ((TimeSleuthMainWindow*)parent())->document;
}

void MainWidget::initTable()
{
    tblTaskList = new TSTable( this, "tblTaskList" );
    tblTaskList->setGeometry( QRect( 0, 0, 236, 190 ) ); 
    tblTaskList->setNumRows( 0 );
    tblTaskList->setNumCols( 0 );

    tblTaskList->blockSignals(true);

    // Set the selection mode
    tblTaskList->setSelectionMode(QTable::NoSelection);

    // Hide left header of task list
    tblTaskList->setLeftMargin(0);
    tblTaskList->verticalHeader()->hide();

    // Set number of table columns
    tblTaskList->setNumCols(3);

    // Set top header of task list
    QHeader* pHeader = tblTaskList->horizontalHeader();
    pHeader->setLabel(0, tr("Start"));
    pHeader->setLabel(1, tr("Stop"));
    pHeader->setLabel(2, tr("Activity"));

    tblTaskList->blockSignals(false);
}

