/*
 * $Id: Synchronize.cpp,v 1.1 2002/10/31 20:06:18 robert Exp $
 *
 * Copyright (C) 2002 Dafydd Walters <dwalters@users.sourceforge.net>
 * 
 * This program may be modified or redistributed under the terms of the GNU
 * General Public License (GPL), version 2 or later. A copy of the GNU GPL 
 * should have accompanied the source code of this program. If not, please 
 * write to the Free Software Foundation Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * This program is distributed without any warranty.
 */ 

#include "Synchronize.h"
#include "TimeSleuthDocument.h"

#include <stdlib.h>
#include <qradiobutton.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <qsocket.h>
#include <qaccel.h>
#include <qapplication.h>
#include <qlabel.h>
#include <qtimer.h>
#include <qtextview.h>

Synchronize::Synchronize(TimeSleuthDocument* doc, QWidget* parent, 
    const char* name, WFlags fl) : 
    SynchronizeBase(parent, name, fl), document(doc)
{
    socket = new QSocket(this, "socket");
    state = WaitingForStart;
    complete = false;
    
    accel = new QAccel(this, "accel");
    accel->insertItem(Qt::Key_Escape, 200);
    accel->connectItem(200, this, SLOT(close()));

    connect(socket, SIGNAL(connected()), this, SLOT(socketConnected()));
    connect(socket, SIGNAL(hostFound()), this, SLOT(socketHostFound()));
    connect(socket, SIGNAL(connectionClosed()), this, 
            SLOT(socketConnectionClosed()));
    connect(socket, SIGNAL(readyRead()), this, SLOT(socketReadyRead()));
    connect(socket, SIGNAL(error(int)), this, SLOT(socketError(int)));
    

    timer = new QTimer(this, 0);
    timer->start(SYNC_TIMEOUT, true);
    connect(timer, SIGNAL(timeout()), this, SLOT(socketTimeout()));
    lblStatus->setText(tr("Looking Up Host"));
    
    connectTime = QDateTime::currentDateTime();
    socket->connectToHost(document->configuration.serverAddress(), 80);
}

Synchronize::~Synchronize()
{
}

void Synchronize::socketError(int e)
{
    switch (e)
    {
        case QSocket::ErrConnectionRefused :
            message(tr("Host Connection Refused"), false);
            break;

        case QSocket::ErrHostNotFound :
            message(tr("Host Not Found"), false);
            break;

        case QSocket::ErrSocketRead :
            message(tr("Socket Read Error"), false);
            break;
    }
}

void Synchronize::socketTimeout()
{
    message(tr("Connection Timed Out"), false);
}

void Synchronize::socketReadyRead()
{
    timer->start(SYNC_TIMEOUT, true);
    while (socket->canReadLine())
    {
        QString l = socket->readLine();

        switch(state)
        {
            case WaitingForStart :
                if (l == "--start--\n")
                {
                    lblStatus->setText(tr("Processing Server Response"));
                    data += l;
                    state = WaitingForEnd;
                }
                break;

            case WaitingForEnd :
                data += l;
                if (l == "--end--\n")
                    state = WaitingForMsg;
                break;

            case WaitingForMsg :
                if (l == "--sync-end--\n")
                    state = Complete;
                else
                    msg += l;
                break;
        }        

        if (state == Complete)
        {
            txtMessages->setText(msg);
            txtMessages->setEnabled(true);
            
            char* byteData = (char*) data.latin1();
    
            if (document->populateFromByteArray(byteData, strlen(byteData)))
            {
                document->tasks.sort();
                document->activities.sort();
                document->activityCategories.sort();
                message(tr("Synchronization Complete"), true);
            }
            else
                message(tr("Synchronization Failed"), false);
            
            break;
        }
    }
}

void Synchronize::message(QString msg, bool success)
{
    timer->stop();
    socket->blockSignals(true);
    socket->close();
    lblStatus->setText(msg);
    QPalette pal = lblStatus->palette();
    QColorGroup cg = lblStatus->palette().active();
    if (success)
    {
        cg.setColor(QColorGroup::Background, QColor(128, 255, 128));
        document->save();
    }
    else
        cg.setColor(QColorGroup::Background, QColor(255, 128, 128));
    pal.setActive( cg );
    lblStatus->setPalette(pal);
    complete = success;
}

void Synchronize::socketConnectionClosed()
{
    message(tr("Synchronization Failed"), false);
}

void Synchronize::socketHostFound()
{
    lblStatus->setText(tr("Host Found"));
    timer->start(SYNC_TIMEOUT, true);
}

void Synchronize::socketConnected()
{
    timer->start(SYNC_TIMEOUT, true);
    lblStatus->setText(tr("Connected"));
    
    QString body = "--sync--\n";
    body += QString::number(connectTime.date().year()) + "\n";
    body += QString::number(connectTime.date().month()) + "\n";
    body += QString::number(connectTime.date().day()) + "\n";
    body += QString::number(connectTime.time().hour()) + "\n";
    body += QString::number(connectTime.time().minute()) + "\n";
    body += QString::number(connectTime.time().second()) + "\n";
    body += document->configuration.serverAddress() + "\n";
    body += document->configuration.serverUserID() + "\n";
    body += document->configuration.serverPassword() + "\n";
    body += document->getDocumentString();
    
    QString request = "POST /palmtopsync/sync";
    request += QString::number(DOCUMENT_VERSION) + ".php3?uid=";
    request += QString::number(rand());
    request += " HTTP/1.0\r\n";
    request += "User-Agent: timesleuthqpe\r\n";
    request += "Content-Type: text/plain\r\n";
    request += "Content-Length: " + QString::number(strlen(body.utf8()));
    request += "\r\n";
    request += "\r\n";
    request += body;
    
    socket->writeBlock(request.utf8(), strlen(request.utf8()));
}

void Synchronize::hide()
{
    socket->close();
    SynchronizeBase::hide();
    qApp->exit_loop();
}

