/*
 * $Id: TaskEdit.cpp,v 1.1 2002/10/31 20:06:18 robert Exp $
 *
 * Copyright (C) 2002 Dafydd Walters <dwalters@users.sourceforge.net>
 * 
 * This program may be modified or redistributed under the terms of the GNU
 * General Public License (GPL), version 2 or later. A copy of the GNU GPL 
 * should have accompanied the source code of this program. If not, please 
 * write to the Free Software Foundation Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * This program is distributed without any warranty.
 */ 

#include "TaskEdit.h"
#include "TimeSleuthDocument.h"
#include "Activity.h"
#include "ActivityEdit.h"
#include "BalloonTip.h"

#include <qlineedit.h>
#include <qapplication.h>
#include <qpe/datebookmonth.h>
#include <qpe/resource.h>
#include <qpe/timestring.h>
#include <qtoolbutton.h>
#include <qpopupmenu.h>
#include <qcombobox.h>
#include <qgroupbox.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qmessagebox.h>
#include <qspinbox.h>
#include <qcheckbox.h>


TaskEdit::TaskEdit(TSTask* t, QWidget* parent, const char* name, 
    bool modal, WFlags fl) : TaskEditBase(parent, name, modal, fl), 
    activity(0), currentTask(t), startDateTime(QDateTime()),
    stopDateTime(QDateTime()), activeList(0)
{
    setupTask();
    setupActivities();
    setupDatePicker();
    setupBalloonTip();

    connect(btnStartDate,SIGNAL(pressed()),this,SLOT(pickStartDate()));
    connect(btnStopDate,SIGNAL(pressed()),this,SLOT(pickStopDate()));
    connect(comboActivity, SIGNAL(activated(int)), this, 
            SLOT(activityActivated(int)));
    connect(spinStartHour, SIGNAL(valueChanged(int)), this, 
            SLOT(startHourChanged(int)));
    connect(spinStartMinute, SIGNAL(valueChanged(int)), this, 
            SLOT(startMinuteChanged(int)));
    connect(spinStopHour, SIGNAL(valueChanged(int)), this, 
            SLOT(stopHourChanged(int)));
    connect(spinStopMinute, SIGNAL(valueChanged(int)), this, 
            SLOT(stopMinuteChanged(int)));
    connect(radStartSpecified, SIGNAL(stateChanged(int)), this, 
            SLOT(startSpecifiedStateChanged(int)));
    connect(radStopSpecified, SIGNAL(stateChanged(int)), this, 
            SLOT(stopSpecifiedStateChanged(int)));
    connect(chkPMStart, SIGNAL(stateChanged(int)), this, 
            SLOT(startPMChanged(int)));
    connect(chkPMStop, SIGNAL(stateChanged(int)), this, 
            SLOT(stopPMChanged(int)));
    connect(btnActivity, SIGNAL(clicked()), this, SLOT(maintainActivities()));
}

TaskEdit::~TaskEdit()
{
    if (activeList)
    {
        activeList->clear();
        delete activeList;
    }
}

void TaskEdit::setupBalloonTip()
{
    BalloonTip::showTip(this, tr("Select an activity, then press OK.\n"
                        "This will start a new task."), 
                        QPoint(20, 60), 5, 1, 1, "timesleuthqpe", 
                        "BalloonTips", "newTask");
}

void TaskEdit::setupTask()
{
    now = TimeSleuthMainWindow::getTSTime();
    if (currentTask)
    {
        setStartDateTime(currentTask->start());
        setStopDateTime(currentTask->stop());
        radStartSpecified->setChecked(true);
        startSpecifiedStateChanged(true);
        if (currentTask->stop().isValid())
        {
            radStopSpecified->setChecked(true);
            stopSpecifiedStateChanged(true);
        }
        activity = currentTask->activity;
    }
    else
    {
        setupTimes();
    }
}

QDateTime TaskEdit::getStartDateTime() 
{
    return startDateTime;
}

QDateTime TaskEdit::getStopDateTime() 
{
    return stopDateTime;
}

TimeSleuthDocument* TaskEdit::document() 
{
    return ((TimeSleuthMainWindow*)parent())->document;
}

void TaskEdit::setupDatePicker()
{
    popup = new QPopupMenu( this );
    picker = new DateBookMonth( popup, 0, TRUE );
    popup->insertItem( picker );
}

void TaskEdit::startPMChanged(int state)
{
    if (document()->configuration.twelveHourClock())
    {
        int hour;
        if (state)
        {
            // PM
            hour = (spinStartHour->value() == 12) ? 12 : 
                    spinStartHour->value() + 12;
        }
        else
        {
            // AM
            hour = (spinStartHour->value() == 12) ? 0 : 
                    spinStartHour->value();
        }
        startDateTime = QDateTime(startDateTime.date(), 
                              QTime(hour, startDateTime.time().minute()));
    }
}

void TaskEdit::stopPMChanged(int state)
{
    if (document()->configuration.twelveHourClock())
    {
        int hour;
        if (state)
        {
            // PM
            hour = (spinStopHour->value() == 12) ? 12 : 
                    spinStopHour->value() + 12;
        }
        else
        {
            // AM
            hour = (spinStopHour->value() == 12) ? 0 : 
                    spinStopHour->value();
        }
        stopDateTime = QDateTime(stopDateTime.date(), 
                              QTime(hour, stopDateTime.time().minute()));
    }
}

void TaskEdit::startSpecifiedStateChanged(int state)
{
    if (state)
    {
        btnStartDate->setEnabled(true);
        spinStartHour->setEnabled(true);
        spinStartMinute->setEnabled(true);
        chkPMStart->setEnabled(true);
    }
    else
    {
        btnStartDate->setEnabled(false);
        spinStartHour->setEnabled(false);
        spinStartMinute->setEnabled(false);
        chkPMStart->setEnabled(false);
    }
}

void TaskEdit::stopSpecifiedStateChanged(int state)
{
    if (state)
    {
        btnStopDate->setEnabled(true);
        spinStopHour->setEnabled(true);
        spinStopMinute->setEnabled(true);
        chkPMStop->setEnabled(true);
    }
    else
    {
        btnStopDate->setEnabled(false);
        spinStopHour->setEnabled(false);
        spinStopMinute->setEnabled(false);
        chkPMStop->setEnabled(false);
    }
}

void TaskEdit::startHourChanged(int value)
{
    int hour;
    if (document()->configuration.twelveHourClock())
    {
        if (chkPMStart->isChecked())
        {
            // PM
            hour = (value == 12) ? 12 : value + 12;
        }
        else
        {
            // AM
            hour = (value == 12) ? 0 : value;
        }
    }
    else
    {
        hour = value;
        if (value < 10)
            spinStartHour->setPrefix("0");
        else
            spinStartHour->setPrefix("");
    }

    startDateTime = QDateTime(startDateTime.date(), 
                              QTime(hour, startDateTime.time().minute()));
}

void TaskEdit::startMinuteChanged(int value)
{
    startDateTime = QDateTime(startDateTime.date(), 
                              QTime(startDateTime.time().hour(), value));
    if (value < 10)
        spinStartMinute->setPrefix("0");
    else
        spinStartMinute->setPrefix("");
}

void TaskEdit::stopHourChanged(int value)
{
    int hour;
    if (document()->configuration.twelveHourClock())
    {
        if (chkPMStop->isChecked())
        {
            // PM
            hour = (value == 12) ? 12 : value + 12;
        }
        else
        {
            // AM
            hour = (value == 12) ? 0 : value;
        }
    }
    else
    {
        hour = value;
        if (value < 10)
            spinStopHour->setPrefix("0");
        else
            spinStopHour->setPrefix("");
    }

    stopDateTime = QDateTime(stopDateTime.date(), 
                              QTime(hour, stopDateTime.time().minute()));
}

void TaskEdit::stopMinuteChanged(int value)
{
    stopDateTime = QDateTime(stopDateTime.date(), 
                              QTime(stopDateTime.time().hour(), value));
    if (value < 10)
        spinStopMinute->setPrefix("0");
    else
        spinStopMinute->setPrefix("");
}

void TaskEdit::setupTimes()
{
   setStartDateTime(now);
   setStopDateTime(now);
}


void TaskEdit::activityActivated(int index)
{
    activity = activeList->at(index);
}

void TaskEdit::setupActivities()
{
    if (document()->configuration.mode() == Configuration::GROUP_MODE)
        btnActivity->hide();
    else if (document()->getNumActiveCategories() == 0)
        btnActivity->setEnabled(false);
   
    if (!activeList)
        activeList = new QList<Activity>;

    activeList->clear();
    comboActivity->clear();

    QListIterator<Activity> li(document()->activities);
    bool found = false;
    for (int i = 0; li.current(); ++li)
    {
        Activity* a = li.current();
        if ((!a->deleted) || (activity == a))
        {
            // Only include activities that have not been deleted, 
            // or one that has been deleted but this task is still 
            // associated with it (cannot choose a deleted activity 
            // for a New task, though).
            activeList->append(a);
            if (a->deleted)
                comboActivity->insertItem(Resource::loadPixmap( "trash" ),
                                          a->name, i);
            else
                comboActivity->insertItem(a->name, i);
            if (activity == a)
            {
                comboActivity->setCurrentItem(i);
                found = true;
            }
	    
            ++i;
        }
    }

    if (!found)
    {
        activity = activeList->first();
        comboActivity->setCurrentItem(0);
    }
}

void TaskEdit::maintainActivities()
{
    ActivityEdit dlg(0, document(), this, 0, true);
    dlg.setCaption(tr("TimeSleuth - New Activity"));
    dlg.showMaximized();
    if (dlg.exec())
    {
        Activity* a = new Activity(document()->allocateId(), 
            dlg.txtName->text().stripWhiteSpace(), dlg.category);
        document()->activities.append(a);
        activity = a;
        document()->activities.sort();
        setupActivities();
    }
}

/*
 * Override accept() virtual function to perform validation
 */
void TaskEdit::accept()
{
    // now = current local time
    now = TimeSleuthMainWindow::getTSTime();

    QDateTime currentStart = radStartNow->isChecked() ? 
                             now : startDateTime;

    QDateTime currentStop;
    if (radStopNow->isChecked())
        currentStop = now;
    else if (radStopLater->isChecked())
        currentStop = currentStart.addSecs(60);
    else
    {
        // specific stop time
        currentStop = stopDateTime;
    }

    switch (document()->validateTask(currentStart, currentStop, now, currentTask))
    {
        case TimeSleuthDocument::STOP_BEFORE_START:

            QMessageBox::warning(this, tr("Time Validation Error"),
                                 tr("The stop time you have selected\n"
                                 "is before the start time"),
                                 QMessageBox::Ok, 0);
            return;

        case TimeSleuthDocument::ZERO_TASK_DURATION:

            QMessageBox::warning(this, tr("Time Validation Error"),
                                 tr("Tasks of zero duration are not\n"
                                 "valid"),
                                 QMessageBox::Ok, 0);
            return;

        case TimeSleuthDocument::TASKS_OVERLAP:
    
            QMessageBox::warning(this, tr("Time Validation Error"),
                        tr("This task is not valid as entered\n"
                        "as it overlaps with an existing\n"
                        "task, and concurrent tasks are\n"
                        "prohibited"), 
                        QMessageBox::Ok, 0);
            return;
    }
    
    TaskEditBase::accept();
}

void TaskEdit::pickStartDate()
{
    picker->disconnect();
    popup->disconnect();
    connect( picker, SIGNAL( dateClicked( int, int, int ) ),
        this, SLOT( setStartDate( int, int, int ) ) );
    connect( popup, SIGNAL( aboutToHide() ),
        this, SLOT( gotHideStartDate() ) );
    picker->setDate(startDateTime.date());
    popup->popup(mapToGlobal(groupStopTime->pos()));
    picker->setFocus();
}

void TaskEdit::pickStopDate()
{
    picker->disconnect();
    popup->disconnect();
    connect( picker, SIGNAL( dateClicked( int, int, int ) ),
            this, SLOT( setStopDate( int, int, int ) ) );
    connect( popup, SIGNAL( aboutToHide() ),
            this, SLOT( gotHideStopDate() ) );
    picker->setDate(stopDateTime.date());
    popup->popup(mapToGlobal(groupActivity->pos()));
    picker->setFocus();
}

void TaskEdit::setStartDateTime(QDateTime dt)
{
    dt = dt.isValid() ? dt : now;
    btnStartDate->setText(TimeString::shortDate(dt.date()));
    if (document()->configuration.twelveHourClock())
    {
        chkPMStart->show();
        if (dt.time().hour() >= 12)
            chkPMStart->setChecked(true);
        else
            chkPMStart->setChecked(false);
        spinStartHour->setPrefix("");
        spinStartHour->setValue((dt.time().hour() % 12) == 0 ? 12 :
                                (dt.time().hour() % 12));
        spinStartHour->setMinValue(1);
        spinStartHour->setMaxValue(12);
    }
    else
    {
        chkPMStart->hide();
        if (dt.time().hour() < 10)
            spinStartHour->setPrefix("0");
        else
            spinStartHour->setPrefix("");
        spinStartHour->setValue(dt.time().hour());
        spinStartHour->setMinValue(0);
        spinStartHour->setMaxValue(23);
    }

    if (dt.time().minute() < 10)
        spinStartMinute->setPrefix("0");
    else
        spinStartMinute->setPrefix("");
    spinStartMinute->setValue(dt.time().minute());

    startDateTime = dt;
}

void TaskEdit::setStopDateTime(QDateTime dt)
{
    dt = dt.isValid() ? dt : now;
    btnStopDate->setText(TimeString::shortDate(dt.date()));
    if (document()->configuration.twelveHourClock())
    {
        chkPMStop->show();
        if (dt.time().hour() >= 12)
            chkPMStop->setChecked(true);
        else
            chkPMStop->setChecked(false);
        spinStopHour->setPrefix("");
        spinStopHour->setValue((dt.time().hour() % 12) == 0 ? 12 : 
                               (dt.time().hour() % 12));
        spinStopHour->setMinValue(1);
        spinStopHour->setMaxValue(12);
    }
    else
    {
        chkPMStop->hide();
        if (dt.time().hour() < 10)
            spinStopHour->setPrefix("0");
        else
            spinStopHour->setPrefix("");
        spinStopHour->setValue(dt.time().hour());
        spinStopHour->setMinValue(0);
        spinStopHour->setMaxValue(23);
    }

    if (dt.time().minute() < 10)
        spinStopMinute->setPrefix("0");
    else
        spinStopMinute->setPrefix("");
    spinStopMinute->setValue(dt.time().minute());

    stopDateTime = dt;
}

void TaskEdit::setStartDate( int y, int m, int d )
{
    startDateTime.setDate(QDate(y, m, d));
    btnStartDate->setText(TimeString::shortDate(
                          startDateTime.date()));
}

void TaskEdit::setStopDate( int y, int m, int d )
{
    stopDateTime.setDate(QDate(y, m, d));
    btnStopDate->setText(TimeString::shortDate(stopDateTime.date()));
}

void TaskEdit::gotHideStartDate()
{
    btnStartDate->setDown( false );
}

void TaskEdit::gotHideStopDate()
{
    btnStopDate->setDown( false );
}

