/*
 * $Id: TimeSleuthDocument.cpp,v 1.1 2002/10/31 20:06:18 robert Exp $
 *
 * Copyright (C) 2002 Dafydd Walters <dwalters@users.sourceforge.net>
 * 
 * This program may be modified or redistributed under the terms of the GNU
 * General Public License (GPL), version 2 or later. A copy of the GNU GPL 
 * should have accompanied the source code of this program. If not, please 
 * write to the Free Software Foundation Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * This program is distributed without any warranty.
 */ 

#include "TimeSleuthDocument.h"
#include "TimeSleuthMainWindow.h"	// TODO remove this
#include <qfile.h>
#include <qpe/global.h>
#include <stdlib.h>
#include <errno.h>

TimeSleuthDocument::TimeSleuthDocument()
{
    nextId = 1000;
}

TimeSleuthDocument::~TimeSleuthDocument()
{
    clear();
}

void TimeSleuthDocument::clear()
{
    // Delete tasks
    TSTask* task = tasks.first();
    while(task)
    {
        delete task;
        task = tasks.next();
    }
    tasks.clear();

    // Delete activities
    Activity* activity = activities.first();
    while(activity)
    {
        delete activity;
        activity = activities.next();
    }
    activities.clear();

    // Delete activity categories
    ActivityCategory* activityCategory = activityCategories.first();
    while(activityCategory)
    {
        delete activityCategory;
        activityCategory = activityCategories.next();
    }
    activityCategories.clear();

}

void TimeSleuthDocument::includeShortcut(TSTask* taskToAdd)
{
    if (taskToAdd->activity->deleted)
        // If activity is obsolete, don't add to shortcuts
        return;
    
    // First check if this task already has a shortcut
    if (taskToAdd->shortcutPos)
    {
        // Nothing to do
        #ifndef NO_DEBUG
            qDebug("Reusing existing shortcut %d for same task", 
                   taskToAdd->shortcutPos);
        #endif
        return;
    }

    TSTask* oldestClosedStopTask = 0;
    TSTask* oldestActiveStartTask = 0;

    // Next, check if there's another task with a shortcut that has
    // the same activity.
    bool buttons[NUM_TASK_SHORTCUTS];
    memset(buttons, 0, NUM_TASK_SHORTCUTS * sizeof(bool));

    QListIterator<TSTask> tli(tasks);
    for( ; tli.current(); ++tli)
    {
        TSTask *task = tli.current();
        
        if (task->shortcutPos >= 1 && task->shortcutPos <= NUM_TASK_SHORTCUTS)
        {
            buttons[task->shortcutPos - 1] = true;

            if (task->stop().isValid())
            {
                // Task has stopped
                if (!oldestClosedStopTask || 
                    (task->stop() < oldestClosedStopTask->stop()))
                {
                    oldestClosedStopTask = task;
                }
            }
            else
            {
                // Task is active
                if (!oldestActiveStartTask || 
                    (task->start() < oldestActiveStartTask->start()))
                {
                    oldestActiveStartTask = task;
                }
            }
        }

        if ((task->activity == taskToAdd->activity) && (task->shortcutPos) &&
            (task != taskToAdd))
        {
            // Existing task with same activity has a shortcut button
            // so reuse this position.
            taskToAdd->shortcutPos = task->shortcutPos;
            taskToAdd->modified();
            task->shortcutPos = 0;
            task->modified();
            #ifndef NO_DEBUG
                qDebug("reusing shortcut position %d", taskToAdd->shortcutPos);
            #endif
            return;
        }
    }

    // Look for an unused shortcut
    for (int i = 0; i < NUM_TASK_SHORTCUTS; i++)
    {
        if (!buttons[i])
        {
            taskToAdd->shortcutPos = i + 1;
            taskToAdd->modified();
            return;
        }
    }
    
    // Re-use oldest closed task
    if (oldestClosedStopTask)
    {
        taskToAdd->shortcutPos = oldestClosedStopTask->shortcutPos;
        taskToAdd->modified();
        oldestClosedStopTask->shortcutPos = 0;
        oldestClosedStopTask->modified();
        #ifndef NO_DEBUG
            qDebug("reusing closed shortcut position %d", taskToAdd->
                   shortcutPos);
        #endif
        return;
    }

    if (oldestActiveStartTask)
    {
        // Last resortL use oldest open task
        taskToAdd->shortcutPos = oldestActiveStartTask->shortcutPos;
        taskToAdd->modified();
        oldestActiveStartTask->shortcutPos = 0;
        oldestActiveStartTask->modified();
        #ifndef NO_DEBUG
            qDebug("reusing active shortcut position %d", taskToAdd->
                   shortcutPos);
        #endif
        return;
    }

    // Should NEVER reach this point
    #ifndef NO_DEBUG
        qDebug("Error allocating shortcut position");
    #endif
}

void TimeSleuthDocument::createSampleData()
{
    clear();

    ActivityCategory* c1 = new ActivityCategory(allocateId(), "Bill rate 1");
    ActivityCategory* c2 = new ActivityCategory(allocateId(), "Bill rate 2");
    ActivityCategory* c3 = new ActivityCategory(allocateId(), "Private");
    activityCategories.append(c1);
    activityCategories.append(c2);
    activityCategories.append(c3);

    activities.append(new Activity(allocateId(), 
        tr("Consulting for client J. Jones"), c1));
    activities.append(new Activity(allocateId(), 
        tr("Project Alpha Development"), c2));
    activities.append(new Activity(allocateId(), 
        tr("Project Alpha Documentation"), c1));
    activities.append(new Activity(allocateId(), tr("Home Project Beta"), c3));
    activities.append(new Activity(allocateId(), tr("Home Project Delta"), c3));

    activityCategories.sort();
    activities.sort();
}

bool TimeSleuthDocument::load()
{
    clear();

    QString fileName = Global::applicationFileName("timesleuthqpe", 
                                                   "timesleuthqpe.dat"); 
    QFile f(fileName);
    if (!f.open(IO_ReadOnly))
        return false;

    QByteArray doc = f.readAll();
    f.close();

    char* dt = doc.data();
    int len = doc.size();
    
    return populateFromByteArray(dt, len);
}

bool TimeSleuthDocument::populateFromByteArray(char* dt, int len)
{

    // Start
    if (readString(&dt, &len) != QString("--start--"))
        return false;
    
    // Read document version
    int i = readInt(&dt, &len);
    if (i != DOCUMENT_VERSION)
        return false;

    // nextId
    nextId = readInt(&dt, &len);

    // Categories
    int j;
    i = readInt(&dt, &len);
    for (j = 0; j < i; j++)
    {
        int id = readInt(&dt, &len);
        QString name = readString(&dt, &len);
        bool deleted = (bool)readInt(&dt, &len);
        ActivityCategory* category = new ActivityCategory(id, name, deleted);
        activityCategories.append(category);
    }

    // Activities
    i = readInt(&dt, &len);
    for (j = 0; j < i; j++)
    {
        int id = readInt(&dt, &len);
        QString name = readString(&dt, &len);
        bool deleted = (bool)readInt(&dt, &len);
        int categoryIndex = readInt(&dt, &len);
        Activity* activity = new Activity(id, name, 
                      activityCategories.at(categoryIndex), deleted);
        activities.append(activity);
    }

    // Tasks
    i = readInt(&dt, &len);
    for(j = 0; j < i; j++)
    {
        int id = readInt(&dt, &len);
        int activityIndex = readInt(&dt, &len);
        int shortcutPos = readInt(&dt, &len);
        int yearFrom = readInt(&dt, &len);
        int monthFrom = readInt(&dt, &len);
        int dayFrom = readInt(&dt, &len);
        int hourFrom = readInt(&dt, &len);
        int minFrom = readInt(&dt, &len);
        QDateTime start;
        if (yearFrom)
            start = QDateTime(QDate(yearFrom,monthFrom,dayFrom),
                              QTime(hourFrom,minFrom));
        int yearTo = readInt(&dt, &len);
        int monthTo = readInt(&dt, &len);
        int dayTo = readInt(&dt, &len);
        int hourTo = readInt(&dt, &len);
        int minTo = readInt(&dt, &len);
        QDateTime stop;
        if (yearTo)
            stop = QDateTime(QDate(yearTo,monthTo,dayTo),
                             QTime(hourTo,minTo));
        int yearModified = readInt(&dt, &len);
        int monthModified = readInt(&dt, &len);
        int dayModified = readInt(&dt, &len);
        int hourModified = readInt(&dt, &len);
        int minModified = readInt(&dt, &len);
        int secModified = readInt(&dt, &len);
        QDateTime modified;
        if (yearModified)
            modified = QDateTime(QDate(yearModified,monthModified,dayModified),
                    QTime(hourModified,minModified,secModified));
        bool inDatabase = (bool)readInt(&dt, &len);
        TSTask* task = new TSTask(id, start, stop, shortcutPos,
                              activities.at(activityIndex), modified, inDatabase);
        tasks.append(task);
    }

    // Deleted Tasks
    i = readInt(&dt, &len);
    deletedTasks.resize(i);
    for(j = 0; j < i; j++)
        deletedTasks[j] = readInt(&dt, &len);
    
    // Signature
    if (readString(&dt, &len) != QString("--end--"))
        return false;

    removeObsoleteActivitiesAndCategories();
    
    return true;
}

/*
 *  Remove any obsolete activities and categories if they are not being used.
 */
void TimeSleuthDocument::removeObsoleteActivitiesAndCategories()
{
    QList<ActivityCategory> deletedCategories;
    deletedCategories.setAutoDelete(true);
    QListIterator<Activity> ali(activities);
    QList<Activity> deletedActivities;
    deletedActivities.setAutoDelete(true);
    for( ali.toFirst(); ali.current(); ++ali)
    {
        Activity* a = ali.current();
        if ((a->deleted) && (!hasActivityGotTasks(a)))
            deletedActivities.append(a);
    }
    Activity* a = deletedActivities.first();
    while(a)
    { 
        activities.removeRef(a);
        a = deletedActivities.next();
    }
    QListIterator<ActivityCategory> cli(activityCategories);
    for( ; cli.current(); ++cli)
    {
        ActivityCategory* c = cli.current();
        if (c->deleted)
        {
            bool found = false;
            for( ali.toFirst(); ali.current(); ++ali)
            {
                Activity* a = ali.current();
                if (a->category == c)
                {
                    found = true;
                    break;
                }
            }
            if (!found)
                deletedCategories.append(c);
        }
    }
    ActivityCategory* c = deletedCategories.first();
    while(c)
    {
        activityCategories.removeRef(c);
        c = deletedCategories.next();
    }
}

void TimeSleuthDocument::skipControlCharacters(char** pdt, int* sizeRemaining)
{
    int i = 0;

    while ((i < *sizeRemaining) && ((unsigned char)(**pdt) < 32))
    {
        // Skip any control characters
        i++;
        (*pdt)++;
    }

    *sizeRemaining -= i;
}

int TimeSleuthDocument::readInt(char** pdt, int* sizeRemaining)
{
    int i = 0;
    int result = 0;

    skipControlCharacters(pdt, sizeRemaining);

    if (i < *sizeRemaining)
    {
        while ((i < *sizeRemaining) && (*(*pdt + i) != '\n'))
        {
            // Look for end of line or end of buffer
            i++;
        }
        if (i < *sizeRemaining)
        {
            *(*pdt + i) = '\0';
            result = atoi(*pdt);
        }
        else
        {
            // reached end of file
            char* temp = (char*)malloc(i + 1);
            strncpy(temp, *pdt, i);
            temp[i] = '\0';
            result = atoi(temp);
            free(temp);
        }
    }

    *pdt += i;
    *sizeRemaining -= i;

    return result; 
}

QString TimeSleuthDocument::readString(char** pdt, int* sizeRemaining)
{
    int i = 0;
    QString result;

    skipControlCharacters(pdt, sizeRemaining);

    if (i < *sizeRemaining)
    {
        while ((i < *sizeRemaining) && (*(*pdt + i) != '\n'))
        { 
            // Look for end of line or end of buffer
            i++;
        }
        result = QString::fromUtf8(*pdt, i);
    }

    *pdt += i;
    *sizeRemaining -= i;
    
    return result; 
}

QString TimeSleuthDocument::getDocumentString()
{
    QString doc;

    // Start
    doc += "--start--\n";
    
    // Version
    doc += QString::number(DOCUMENT_VERSION) + "\n";

    // nextId
    doc += QString::number(nextId) + "\n";

    // Categories
    doc += QString::number(activityCategories.count()) + "\n";
    ActivityCategory* category = activityCategories.first();
    while (category)
    {
        doc += QString::number(category->id) + "\n";
        doc += category->name + "\n";
        doc += QString::number(category->deleted) + "\n";
        category = activityCategories.next();
    }

    // Activities
    doc += QString::number(activities.count()) + "\n";
    Activity* activity = activities.first();
    while (activity)
    {
        doc += QString::number(activity->id) + "\n";
        doc += activity->name + "\n";
        doc += QString::number(activity->deleted) + "\n";
        doc += QString::number(activityCategories.findRef(activity->category));
        doc += "\n";
        activity = activities.next();
    }

    // Tasks
    doc += QString::number(tasks.count()) + "\n";
    TSTask* task = tasks.first();
    while (task)
    {
        doc += QString::number(task->id) + "\n";
        doc += QString::number(activities.findRef(task->activity)) + "\n";
        doc += QString::number(task->shortcutPos) + "\n";
        if (!task->start().isValid())
        {
            doc += "0\n0\n0\n0\n0\n";
        }
        else
        {
            doc += QString::number(task->start().date().year()) + "\n";
            doc += QString::number(task->start().date().month()) + "\n";
            doc += QString::number(task->start().date().day()) + "\n";
            doc += QString::number(task->start().time().hour()) + "\n";
            doc += QString::number(task->start().time().minute()) + "\n";
        }
        if (!task->stop().isValid())
        {
            doc += "0\n0\n0\n0\n0\n";
        }
        else
        {
            doc += QString::number(task->stop().date().year()) + "\n";
            doc += QString::number(task->stop().date().month()) + "\n";
            doc += QString::number(task->stop().date().day()) + "\n";
            doc += QString::number(task->stop().time().hour()) + "\n";
            doc += QString::number(task->stop().time().minute()) + "\n";
        }
        if (!task->lastModified.isValid())
        {
            doc += "0\n0\n0\n0\n0\n0\n";
        }
        else
        {
            doc += QString::number(task->lastModified.date().year()) + "\n";
            doc += QString::number(task->lastModified.date().month()) + "\n";
            doc += QString::number(task->lastModified.date().day()) + "\n";
            doc += QString::number(task->lastModified.time().hour()) + "\n";
            doc += QString::number(task->lastModified.time().minute()) + "\n";
            doc += QString::number(task->lastModified.time().second()) + "\n";
        }
        doc += QString::number(task->inDatabase) + "\n";
        task = tasks.next();
    }

    // Deleted Tasks
    doc += QString::number(deletedTasks.size()) + "\n";
    for (unsigned int i = 0; i < deletedTasks.size(); i ++)
        doc += QString::number(deletedTasks[i]) + "\n";
    
    // Signature
    doc += "--end--\n";

    return doc;
}

bool TimeSleuthDocument::save()
{
    QString fileName = Global::applicationFileName("timesleuthqpe", 
                                                   "timesleuthqpe.dat"); 
    QString fileTempName = fileName + ".new";

    QFile f(fileTempName);
    if (!f.open(IO_WriteOnly | IO_Raw))
        return false;

    QCString str;
    int totalWritten;

    str = getDocumentString().utf8();

    totalWritten = f.writeBlock(str.data(), str.length());
    if (totalWritten != int(str.length())) 
    {
        f.close();
        QFile::remove(fileTempName);
        return false;
    }
    f.close();

    // now do the rename
    if (::rename(fileTempName, fileName) < 0)
        qWarning("problem renaming file %s to %s errno %d",
            fileTempName.latin1(), fileName.latin1(), errno);

    return true;
}

int TimeSleuthDocument::allocateId()
{
    return nextId++;
}

bool TimeSleuthDocument::activeTasks()
{
    TSTask* t = tasks.first();
    while (t)
    {
        if (!t->stop().isValid())
            return true;
        t = tasks.next();
    }

    return false;
}

int TimeSleuthDocument::getActivityDuration(QDate fromDate, QDate toDate, 
    Activity* activity, bool* openTasks)
{
    int minutes = 0;
    *openTasks = false;    

    QListIterator<TSTask> tli(tasks);
    for( ; tli.current(); ++tli)
    {
        TSTask* task = tli.current();
        if (task->activity == activity)
        {
            if (task->stop().isValid())
            {
                if (task->start() >= QDateTime(fromDate) &&
                    task->stop() <= QDateTime(toDate).addDays(1))
                {
                    // The entire duration of this task is to be counted
                    minutes += task->duration();
                }
                else if (task->start() < QDateTime(fromDate) &&
                         task->stop() > QDateTime(toDate).addDays(1))
                {
                    // Task spans the entire report period
                    minutes += QDateTime(fromDate).secsTo(QDateTime(
                               toDate).addDays(1)) / 60;
                }
                else if (task->start() < QDateTime(fromDate) &&
                         task->stop() > QDateTime(fromDate) &&
                         task->stop() <= QDateTime(toDate).addDays(1))
                {
                    // Task starts before report period, but ends
                    // within report period
                    minutes += QDateTime(fromDate).secsTo(task->stop()) / 60;
                }
                else if (task->start() < QDateTime(toDate).addDays(1) &&
                         task->start() >= QDateTime(fromDate) &&
                         task->stop() > QDateTime(toDate).addDays(1))
                {
                    // Task starts within report period, but ends
                    // after report period
                    minutes += QDateTime(task->start()).secsTo(QDateTime(
                               toDate).addDays(1)) / 60;
                }
            }
            else
            {
                if (task->start() < QDateTime(toDate).addDays(1))
                {
                    *openTasks = true;
                }
            }
        }
    }

    return minutes;
}

TimeSleuthDocument::ValidationCode TimeSleuthDocument::validateTask(
    QDateTime start, QDateTime stop, QDateTime now, TSTask* currentTask)
{

    // Make sure start time is before stop time
    if (stop < start)
    {
        return STOP_BEFORE_START;
    }
    else if (stop == start)
    {
        return ZERO_TASK_DURATION;
    }

    // If concurrent tasks are disallowed, check here whether 
    // start or stop date/time would cause an illegal
    // time overlap with any other tasks.
    if (!configuration.allowConcurrentTasks())
    {
        // Concurrent tasks are not allowed

        // For the purpose of comparing whether tasks overlap, we need to
        // obtain assumed values for start and stop times for the current task.
        QListIterator<TSTask> tli(tasks);
        for ( ; tli.current(); ++tli)
        {
            TSTask* t = tli.current();
            if ((t != currentTask) && (t->start() != now))
            {
                QDateTime chkStart = t->start();
                QDateTime chkStop;
                if (t->stop().isValid())
                {
                    // task t has stopped; use explicit stop time
                    chkStop = t->stop();
                }
                else
                {
                    // assume the stop time is the shortest possible task
                    // duration
                    chkStop = chkStart.addSecs(60);
                }

                if (((stop > chkStart) && (stop < chkStop)) ||
                    ((start > chkStart) && (start < chkStop)) ||
                    ((chkStart > start) && (chkStart < stop)) ||
                    ((chkStop > start) && (chkStop < stop)) ||
                    ((start == chkStart) && (stop == chkStop) &&
                     (start != stop)))
                {
                    return TASKS_OVERLAP;
                }
            }
        }
    }
    
    return NO_ERRORS;
}

/*
 * Note that removeTask removes the task from the tasks list, but
 * does NOT delete the task object.
 */
void TimeSleuthDocument::removeTask(TSTask* t)
{
    tasks.removeRef(t);

    if (t->inDatabase)
    {
        int i = deletedTasks.size();
        deletedTasks.resize(i + 1);
        deletedTasks[i] = t->id;
    }
    
    if (t->shortcutPos >= 1 && t->shortcutPos <= NUM_TASK_SHORTCUTS)
    {
        // Deleting a task that has a shortcut.  
        QListIterator<TSTask> tli(tasks);
        // Try to find most recent active task with same activity 
        for( tli.toLast(); tli.current(); --tli)
        {
            TSTask* task = tli.current();
            if ((task->activity == t->activity) && (!task->stop().isValid()))
            {
                includeShortcut(task);
                return;
            }
        }
        // Find a closed task with the same activity.
        for( tli.toFirst(); tli.current(); ++tli)
        {
            TSTask* task = tli.current();
            if ((task->activity == t->activity) && task->stop().isValid())
            {
                includeShortcut(task);
                return;
            }
        }
    }
}

void TimeSleuthDocument::stopTask(TSTask* t, QDateTime now)
{
    t->modified();
    t->setStop(now);
    
    if (t->shortcutPos >= 1 && t->shortcutPos <= NUM_TASK_SHORTCUTS)
    {
        // Stopping a task that has a shortcut.  
        QListIterator<TSTask> tli(tasks);
        // Try to find most recent active task with same activity 
        for( tli.toLast(); tli.current(); --tli)
        {
            TSTask* task = tli.current();
            if ((task->activity == t->activity) && (!task->stop().isValid()))
            {
                includeShortcut(task);
                return;
            }
        }
    }
}

/*
 * Returns number of activities which have not been marked for deletion.
 * if includeThoseWithTasks is true, include deleted tasks if they have
 * an existing tasks still associated with them.
 */
int TimeSleuthDocument::getNumActiveActivities(bool includeThoseWithTasks)
{
    int result = 0;
    QListIterator<Activity> li(activities);
    for ( ; li.current(); ++li)
    {
        Activity* a = li.current();
        if (!a->deleted ||
            (includeThoseWithTasks && hasActivityGotTasks(a)))
        {
            result++;
        }
    }
    return result;
}

/*
 * Returns number of categories which have not been marked for deletion.
 */
int TimeSleuthDocument::getNumActiveCategories()
{
    int result = 0;
    QListIterator<ActivityCategory> li(activityCategories);
    for ( ; li.current(); ++li)
    {
        ActivityCategory* c = li.current();
        if (!c->deleted)
            result++;
    }
    return result;
}

bool TimeSleuthDocument::hasActivityGotTasks(Activity* a)
{
    QListIterator<TSTask> li(tasks);
    for (; li.current(); ++li)
    {
        if (li.current()->activity == a)
            return true;
    }
    return false;
}
