/*
 * $Id: TimeSleuthMainWindow.cpp,v 1.1 2002/10/31 20:06:18 robert Exp $
 *
 * Copyright (C) 2002 Dafydd Walters <dwalters@users.sourceforge.net>
 * 
 * This program may be modified or redistributed under the terms of the GNU
 * General Public License (GPL), version 2 or later. A copy of the GNU GPL 
 * should have accompanied the source code of this program. If not, please 
 * write to the Free Software Foundation Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * This program is distributed without any warranty.
 */ 

#include "TimeSleuthMainWindow.h"
#include "MainWidget.h"
#include "TimeSleuthDocument.h"
#include "ChooseInitialMode.h"
#include "TaskEdit.h"
#include "Configuration.h"
#include "PeriodSummaryReport.h"
#include "PeriodTimeSheetReport.h"
#include "PurgeTasks.h"
#include "OperatingMode.h"
#include "ActivityList.h"
#include "ModalWidget.h"
#include "FileSelectDialog.h"
#include "CategoryList.h"
#include "BalloonTip.h"
#include "Synchronize.h"
#include "TSTable.h"

#include <stdlib.h>
#include <time.h>
#include <qpe/fileselector.h>
#include <qpe/config.h>
#include <qsocket.h>
#include <qapplication.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qtable.h>
#include <qheader.h>
#include <qpe/qpemenubar.h>
#include <qpopupmenu.h>
#include <qradiobutton.h>
#include <qtoolbutton.h>
#include <qmessagebox.h>
#include <qtimer.h>
#include <qtextview.h>
#include <qpe/applnk.h>
#include <qpe/filemanager.h>
#include <qstringlist.h>
#include <qpe/qpetoolbar.h>
#include <qpe/resource.h>
#include <qpe/qpemessagebox.h>
#include <qmenubar.h>

TimeSleuthMainWindow::TimeSleuthMainWindow( ) : 
    QMainWindow( 0, "TimeSleuth time tracking application" )
{
    srand((unsigned int)((time(0) + QTime::currentTime().msec()) % RAND_MAX));
    setToolBarsMovable( FALSE );
    setupDocument();
    setupReports();
    setupMenuBar();
    setupCentralWidget();
    setupTimer();
    updateView();
    showBalloonTips();

    connect(view, SIGNAL(taskChanged()), this,  SLOT(taskChanged()));
    connect(view->btnTask1, SIGNAL(stateChanged(int)), this, 
            SLOT(shortcut1Toggled(int)));
    connect(view->btnTask2, SIGNAL(stateChanged(int)), this, 
            SLOT(shortcut2Toggled(int)));
    connect(view->btnTask3, SIGNAL(stateChanged(int)), this, 
            SLOT(shortcut3Toggled(int)));
    connect(timer, SIGNAL(timeout()), view, SLOT(updateShortcutTimers()));
    connect(view->tblTaskList, SIGNAL( ok() ), this, SLOT( editTask() ) );
}

/*  
 *  Destroys the object and frees any allocated resources
 */
TimeSleuthMainWindow::~TimeSleuthMainWindow()
{
    // no need to delete child widgets, Qt does it all for us

    // save data
    if (!document->save())
    {
        #ifndef NO_DEBUG
            qDebug(tr("failed to save document"));
        #endif
    }
    delete document;
}

void TimeSleuthMainWindow::showBalloonTips()
{
    Config config("timesleuthqpe");
    config.setGroup("BalloonTips");

    switch (document->configuration.mode())
    {
        case Configuration::PERSONAL_MODE :
        {
            if (config.readNumEntry("taskList", 0) > 0)
            {
                BalloonTip::showTip(view, tr("For additional help, tap the (?)\n"
                                         "icon at the top of the screen.\n\n"
                                         "This is the end of balloon-tip help."), 
                                    QPoint(20, 70), 7, 1, 1, "timesleuthqpe", 
                                    "BalloonTips", "additionalHelp");
            }
            else if (document->tasks.count() > 0)
            {
                BalloonTip::showTip(view, tr("Tasks are listed here"), 
                                    QPoint(40, 40), 5, 1, 1, "timesleuthqpe", 
                                    "BalloonTips", "taskList");
        
                BalloonTip::showTip(view, 
                                   tr("Use these \"shortcut\" buttons to start\n"
                                   "and stop the three most recent\n"
                                   "activities in real time"), 
                                   QPoint(20, 160), 10, 1, 1, "timesleuthqpe", 
                                   "BalloonTips", "shortcutButtons");
            }
            else if (document->getNumActiveActivities() > 0)
            {
                BalloonTip::showTip(view, 
                                    tr("Tap the New Task icon to\nget started"), 
                                    QPoint(90, 25), 4, 1, 1, "timesleuthqpe", 
                                    "BalloonTips", "startHere");
            }
            else if (document->getNumActiveCategories() > 0)
            {
                BalloonTip::showTip(view, 
                                    tr("Select \"Activities\" from the\n"
                                       "\"Settings\" menu to create some\n"
                                       "new activities"), 
                                    QPoint(30, 25), 6, 1, 1, "timesleuthqpe", 
                                    "BalloonTips", "activities");
            }
            else
            {
                BalloonTip::showTip(view, 
                                    tr("Select \"Categories\" from the\n"
                                       "\"Settings\" menu to create some\n"
                                       "new activity categories"), 
                                    QPoint(30, 25), 6, 1, 1, "timesleuthqpe", 
                                    "BalloonTips", "categories");
            }
            break;        
        }

        case Configuration::GROUP_MODE :
        {
            BalloonTip::showTip(view, 
                                tr("Select \"Synchronize with Server\"\n"
                                   "from the \"Task\" menu to get started\n"), 
                                QPoint(30, 25), 6, 1, 1, "timesleuthqpe", 
                                "BalloonTips", "groupMode");
            break;
        }
    }
}

void TimeSleuthMainWindow::resetBalloonTips()
{
    // Reset balloon tip help
    Config config("timesleuthqpe");
    config.setGroup("BalloonTips");
    config.writeEntry("groupMode", 0);
    config.writeEntry("categories", 0);
    config.writeEntry("activities", 0);
    config.writeEntry("startHere", 0);
    config.writeEntry("newTask", 0);
    config.writeEntry("shortcutButtons", 0);
    config.writeEntry("taskList", 0);
    config.writeEntry("additionalHelp", 0);

    // Reset some preferences to defaults
    config.setGroup("LocalSettings");
    config.writeEntry("WarnDeleteZeroTasks", 1);
    config.writeEntry("AllowConcurrentTasks", 0);
}

void TimeSleuthMainWindow::shortcut1Toggled(int state)
{
    shortcutToggled(1, state, view->btnTask1);
}

void TimeSleuthMainWindow::shortcut2Toggled(int state)
{
    shortcutToggled(2, state, view->btnTask2);
}

void TimeSleuthMainWindow::shortcut3Toggled(int state)
{
    shortcutToggled(3, state, view->btnTask3);
}

void TimeSleuthMainWindow::shortcutToggled(int pos, int state, 
                                           QToolButton* button)
{
    disableButtons();
    
    #ifndef NO_DEBUG
        qDebug("task %d button toggled", pos);
    #endif

    QListIterator<TSTask> tli(document->tasks);
    for( ; tli.current(); ++tli)
    {
        TSTask* t = tli.current();
        
        if (t->shortcutPos == pos)
        {
            if (state == QButton::On)
            {
                    if (t->stop().isValid())
                    {
                        // Need to start a new task
                        QDateTime now = getTSTime();
                        if (document->validateTask(now, now.addSecs(60), now, 0)
                            == TimeSleuthDocument::TASKS_OVERLAP)
                        {

                            QMessageBox::information(this, 
                                 tr("Cannot Start Task"),
                                 tr("This task cannot be started\n"
                                 "now because this would cause it\n"
                                 "to overlap with another task.\n"),
                                 QMessageBox::Ok);
                            button->setOn(false);
                            break;

                        }

                        TSTask* newTask = new TSTask(document->allocateId(), 
                            now, QDateTime(), 0, t->activity);
                        document->tasks.append(newTask);
                        if (!stopOtherTasks(newTask, now, false))
                        {
                            button->setOn(false);
                            document->removeTask(newTask);
                            delete newTask;
                            break;
                        }
                        document->includeShortcut(newTask);
                        view->selectedTask = newTask;
                        document->tasks.sort();
                        updateView();
                    }
                    else
                    {
                        #ifndef NO_DEBUG
                            qDebug("shortcut btn clicked on but "
                                   "already active");
                        #endif
                    }
            }
            else if (state == QButton::Off)
            {
                    if (t->stop().isValid())
                    {
                        #ifndef NO_DEBUG
                            qDebug("shortcut btn clicked off but "
                                   "already closed!");
                        #endif
                    }
                    else
                    {
                        // Need to stop task
                        QDateTime now = getTSTime();
                        switch (document->validateTask(t->start(), now, now, t)) 
                        {
                            case TimeSleuthDocument::STOP_BEFORE_START:

                                QMessageBox::information(this, 
                                 tr("Cannot Stop Task"),
                                 tr("This task cannot be stopped\n"
                                 "now because its start time is in\n"
                                 "the future. To set a specific\n"
                                 "future stop time, you can Edit\n"
                                 "the task."), 
                                 QMessageBox::Ok);
                                button->setOn(true);
                                break;
                         
                            case TimeSleuthDocument::ZERO_TASK_DURATION:
                            {    
                                Config config("timesleuthqpe");
                                config.setGroup("LocalSettings");
                                if (config.readBoolEntry("WarnDeleteZeroTasks", 
                                                         true))
                                {
                                    if (QMessageBox::warning(this, 
                                        tr("Zero Duration Task"),
                                        tr("Tasks of less than 1 minute\n"
                                        "duration are not logged, so\n"
                                        "this task will be deleted.\n\n"
                                        "Would you like to skip this\n"
                                        "warning message in future?"), 
                                        QMessageBox::Yes,
                                        QMessageBox::No|QMessageBox::Default|
                                        QMessageBox::Escape) 
                                        == QMessageBox::Yes)
                                    {
                                        config.writeEntry("WarnDeleteZeroTasks", 
                                                          false);
                                    }
                                }
                                document->removeTask(t);
                                view->selectedTask = 0;
                                updateView();
                                delete t;
                                break;
                            }
                            case TimeSleuthDocument::TASKS_OVERLAP:

                                QMessageBox::information(this, 
                                 tr("Cannot Stop Task"),
                                 tr("This task cannot be stopped\n"
                                 "now because this would cause it\n"
                                 "to overlap with another task.\n"
                                 "You can Edit the task to set\n"
                                 "a stop time."), 
                                 QMessageBox::Ok);
                                button->setOn(true);
                                break;

                            case TimeSleuthDocument::NO_ERRORS:

                                document->stopTask(t, now);
                                view->selectedTask = t;
                                updateView(t);
                                break;
                        }
                    }
            }
            break;
        }
    }

    enableButtons();
    showBalloonTips();
}

void TimeSleuthMainWindow::disableButtons()
{
    view->btnTask1->blockSignals(true);
    view->btnTask2->blockSignals(true);
    view->btnTask3->blockSignals(true);
    view->btnTask1->setEnabled(false);
    view->btnTask2->setEnabled(false);
    view->btnTask3->setEnabled(false);
    qApp->processEvents();
}

void TimeSleuthMainWindow::enableButtons()
{
    view->btnTask1->setEnabled(!view->btnTask1->text().isEmpty());
    view->btnTask2->setEnabled(!view->btnTask2->text().isEmpty());
    view->btnTask3->setEnabled(!view->btnTask3->text().isEmpty());
    view->btnTask1->blockSignals(false);
    view->btnTask2->blockSignals(false);
    view->btnTask3->blockSignals(false);
}

void TimeSleuthMainWindow::setupTimer()
{
    timer = new QTimer(this, "one second timer");
    timer->start(1000);
}


void TimeSleuthMainWindow::taskChanged()
{
    // No need to update view, slot in view already receives
    // the taskChanged signal and updates buttons.
    // Just update menu and toolbar selections.
    updateControls();
}

void TimeSleuthMainWindow::updateView(TSTask* t)
{
    // Make sure view is synchronized with document
    if (t)
    {
        view->updateRow(t);
        view->setSelectedTask();
        view->blankUnusedButtons();
    }
    else
        view->update();

    // Update the shortcut time counters
    view->updateShortcutTimers();    
    
    // Update menu and toolbar buttons
    updateControls();
}

void TimeSleuthMainWindow::updateControls()
{
    if (document->getNumActiveActivities() == 0)
    {
        newAction->setEnabled(false);
    }
    else
    {
        newAction->setEnabled(true);
    }

    if (view->selectedTask)
    {
        editAction->setEnabled(true);
        deleteAction->setEnabled(true);
    }
    else
    {
        editAction->setEnabled(false);
        deleteAction->setEnabled(false);
    }

    if (document->tasks.count() > 0)
    {
        purgeAction->setEnabled(true);
    }
    else
    {
        purgeAction->setEnabled(false);
    }

    if (((document->getNumActiveCategories() == 0) && 
         (document->getNumActiveActivities() == 0)) ||
        (document->configuration.mode() == Configuration::GROUP_MODE))
    {
        activitiesAction->setEnabled(false);
    }
    else
    {
        activitiesAction->setEnabled(true);
    }

    if (document->configuration.mode() == Configuration::GROUP_MODE)
    {
        syncAction->setEnabled(true);
        categoriesAction->setEnabled(false);
    }
    else
    {
        syncAction->setEnabled(false);
        categoriesAction->setEnabled(true);
    }
}

void TimeSleuthMainWindow::deleteTask()
{
    if (!view->selectedTask)
    {
        updateView();
        return;
    }

    TSTask* t = view->selectedTask;

    if (QMessageBox::warning(this, tr("Delete Task"),
             tr("Are you sure you want to delete\n%1?").arg(t->activity->name),
             QMessageBox::Yes, QMessageBox::No | QMessageBox::Default |
             QMessageBox::Escape) != QMessageBox::Yes)
    {
        return;
    }

    document->removeTask(t);
    view->selectedTask = 0;
    updateView();
    delete t;
}

void TimeSleuthMainWindow::editTask()
{
    if (!view->selectedTask)
    {
        updateView();
        return;
    }

    TSTask* t = view->selectedTask;
    QDateTime oldStart = t->start(); 
    QDateTime oldStop = t->stop(); 
    Activity* oldActivity = t->activity; 

    TaskEdit dlg(t, this, 0, true);
    dlg.showMaximized();
    dlg.setCaption(tr("TimeSleuth - Edit Task"));

    if (dlg.exec())
    {
        // OK
        t->modified();
        if (dlg.radStartNow->isChecked())
            t->setStart(dlg.now);
        else if (dlg.radStartSpecified->isChecked())
            t->setStart(dlg.getStartDateTime());

        if (dlg.radStopNow->isChecked())
            t->setStop(dlg.now);
        else if (dlg.radStopSpecified->isChecked())
            t->setStop(dlg.getStopDateTime());
        else if (dlg.radStopLater->isChecked())
            t->setStop(QDateTime()); // null date time	
        t->activity = dlg.activity;    

        if (!stopOtherTasks(t, dlg.now, true))
        {
            t->setStart(oldStart);
            t->setStop(oldStop);
            t->activity = oldActivity;
        }

        bool shortcutsChanged = false;
        if (t->activity != oldActivity)
        {
            // The activity has changed.  Make sure this does not result in
            // a duplicate shortcut legend.
            QListIterator<TSTask> tli(document->tasks);
            for( ; tli.current(); ++tli)
            {
                TSTask* task = tli.current();

                if ((t != task) && (t->activity == task->activity) && 
                    (task->shortcutPos >= 1) && 
                    (task->shortcutPos <= NUM_TASK_SHORTCUTS))
                {
                    task->shortcutPos = 0;
                    task->modified();
                    shortcutsChanged = true;
                }
            }
            
        }

        if (!t->stop().isValid())
            document->includeShortcut(t);
        else if (!oldStop.isValid())
            document->stopTask(t, t->stop());
        
        if ((t->start() != oldStart) || shortcutsChanged)
        {
            document->tasks.sort();
            updateView();
        }
        else
        {
            updateView(t);
        }
    }
}

/*
 * stopOtherTasks()
 *
 * If concurrent tasks are disabled, this function makes sure that
 * only the current tasks is active (stops all others).
 */
bool TimeSleuthMainWindow::stopOtherTasks(TSTask* currentTask, QDateTime now,
                                          bool updateStoppedTasks)
{
    if (!currentTask || currentTask->stop().isValid())
        // No task is currently selected, or current task is stopped
        return true;

    QList<TSTask> deleted;
    deleted.setAutoDelete(true);
    QListIterator<TSTask> tli(document->tasks);
    
    if (document->configuration.allowConcurrentTasks())
    {
        // Concurrent tasks are allowed.  Only look for other active tasks
        // with same activity.
        for( ; tli.current(); ++tli)
        {
            TSTask* t = tli.current();
            if ((t != currentTask) && (t->activity == currentTask->activity)
                && (!t->stop().isValid()))
            {
                QMessageBox::information(this, 
                        tr("Warning"),
                        tr("There are other tasks with\n"
                            "the same activity still\n"
                            "active."), 
                        QMessageBox::Ok);
                return true;
            }
        } 
    }
    else 
    {
        // Concurrent tasks are not allowed,
        // so ensure that all other tasks are stopped.
        for( ; tli.current(); ++tli)
        {
            TSTask* t = tli.current();
            if ((t != currentTask) && (!t->stop().isValid()))
            {
                switch (document->validateTask(t->start(), now, now, t))
                {
                    case TimeSleuthDocument::STOP_BEFORE_START:

                        QMessageBox::warning(this, tr("Time Validation Error"),
                                 tr("Unable to stop another task now, as\n"
                                 "its start time is in the future."),
                                 QMessageBox::Ok, 0);
                        return false;

                    case TimeSleuthDocument::ZERO_TASK_DURATION:

                        document->removeTask(t);
                        deleted.append(t);
                        updateView();
                        break;

                    case TimeSleuthDocument::TASKS_OVERLAP:
    
                        QMessageBox::warning(this, tr("Time Validation Error"),
                            tr("Unable to stop another task now,\n"
                            "as it would cause tasks to overlap."),
                            QMessageBox::Ok, 0);
                        return false;

                    case TimeSleuthDocument::NO_ERRORS:

                        document->stopTask(t, now);
                        if (updateStoppedTasks)
                            updateView(t);
                        break;
                }

            }
        }
    }
    return true;
}

void TimeSleuthMainWindow::setupDocument()
{
    document = new TimeSleuthDocument();

    if ((!document->load()) || (document->configuration.mode() == 
                Configuration::UNDEFINED_MODE))
    {
        resetBalloonTips();
        promptUserMode();
    }
}

void TimeSleuthMainWindow::promptUserMode()
{
    ChooseInitialMode dlg(this, "ChooseInitialMode", true);
    dlg.showMaximized();
    
    bool selectionMade = false;
    while (!selectionMade)
    {
        dlg.show();
        if (!dlg.exec())
        {
            document->configuration.setMode(Configuration::UNDEFINED_MODE);
            exit(0);
        }
        
        if (dlg.radPersonalInit->isChecked())
        {
            document->configuration.setMode(Configuration::PERSONAL_MODE);
            document->createSampleData();
            selectionMade = true;
        }
        else if (dlg.radPersonalNoInit->isChecked())
        {
            document->configuration.setMode(Configuration::PERSONAL_MODE);
            document->clear();
            selectionMade = true;
        }
        else if (dlg.radGroup->isChecked())
        {
            document->configuration.setMode(Configuration::GROUP_MODE);
            document->clear();
            selectionMade = getServerSettings();
        }
    }
}

bool TimeSleuthMainWindow::getServerSettings()
{
    OperatingMode dlg(this, 0, true);
    dlg.showMaximized();
    dlg.txtServer->setText(document->configuration.serverAddress());
    dlg.txtPassword->setText(document->configuration.serverPassword());
    dlg.txtUserID->setText(document->configuration.serverUserID());
    dlg.radGroup->setChecked(true);
    dlg.radPersonal->setEnabled(false);
    
    if (!dlg.exec())
        return false;
    
    document->configuration.setMode(Configuration::GROUP_MODE);
    document->configuration.setServerAddress(dlg.txtServer->text().stripWhiteSpace());
    document->configuration.setServerUserID(dlg.txtUserID->text().stripWhiteSpace());
    document->configuration.setServerPassword(dlg.txtPassword->text().stripWhiteSpace());
    return true;
}

void TimeSleuthMainWindow::setupMenuBar()
{
    setToolBarsMovable(false);

    // Create a menu bar for the text menus
    QPEToolBar *bar = new QPEToolBar( this );
    bar->setHorizontalStretchable( TRUE );
    QPEMenuBar *mb = new QPEMenuBar( bar );

    // Create the menu objects
    QPopupMenu *taskMenu = new QPopupMenu( this );
    QPopupMenu *reportMenu = new QPopupMenu(this);
    QPopupMenu *settingsMenu = new QPopupMenu(this);

    // Create the task popup menu
    contextMenu = new QPopupMenu( this );

    // Create the tool bar for the buttons
    bar = new QPEToolBar( this );

    // insert the action items for the task menu
    addTaskMenuItems(taskMenu, contextMenu, bar);

    // Add the reports to the report menu
    addReportMenuItems(reportMenu);

    // insert the menu items for the settings menu
    addSettingsMenuItems(settingsMenu);

    // Populate menu bar
    mb->insertItem(tr("Task"), taskMenu );
    mb->insertItem(tr("Report"), reportMenu);
    mb->insertItem(tr("Settings"), settingsMenu);
}

void TimeSleuthMainWindow::addTaskMenuItems(QPopupMenu* taskMenu,
                                            QPopupMenu* contextMenu,
                                            QToolBar* bar)
{
    // New task action item
    newAction = new QAction( tr( "New Task" ), Resource::loadPixmap( "new" ),
                             QString::null, 0, this, 0 );
    connect( newAction, SIGNAL( activated() ),
             this, SLOT( newTask() ) );
    newAction->addTo( bar );
    newAction->addTo( taskMenu );

    // Edit task action item
    editAction = new QAction( tr( "Edit Task" ), 
                              Resource::loadPixmap( "edit" ),
                              QString::null, 0, this, 0 );
    connect( editAction, SIGNAL( activated() ),
             this, SLOT( editTask() ) );
    editAction->addTo( bar );
    editAction->addTo( taskMenu );
    editAction->addTo( contextMenu );
    editAction->setEnabled( FALSE );

    // Delete task action item
    deleteAction = new QAction( tr( "Delete Task" ),
                                Resource::loadPixmap( "trash" ),
                                QString::null, 0, this, 0 );
    connect( deleteAction, SIGNAL( activated() ),
             this, SLOT( deleteTask() ) );
    deleteAction->addTo( bar );
    deleteAction->addTo( taskMenu );
    deleteAction->addTo( contextMenu );
    deleteAction->setEnabled( FALSE );

    // Purge
    taskMenu->insertSeparator();
    purgeAction = new QAction(tr("Purge Tasks"),
                              Resource::loadPixmap("editdelete"),
                              QString::null, 0, this, 0);
    purgeAction->addTo(taskMenu);
    connect(purgeAction, SIGNAL(activated()), this, SLOT(purge()));

    // Sync
    syncAction = new QAction(tr("Synchronize with Server"),
                             Resource::loadPixmap("repeat"),
                             QString::null, 0, this, 0);
    syncAction->addTo(taskMenu);
    connect(syncAction, SIGNAL(activated()), this, SLOT(syncServer()));
}

void TimeSleuthMainWindow::addReportMenuItems(QPopupMenu * menu)
{
    QAction* reportAction;
    Report* report = reports.first();
    while(report)
    {    
        reportAction = new QAction( report->getMenuName(), 
                                QString::null, 0, this, 0 );
        reportAction->addTo( menu );
        connect(reportAction, SIGNAL(activated()), report, SLOT(invoke()));
        report = reports.next();
    }

    menu->insertSeparator();

    viewSavedReportAction = new QAction(tr("View Saved Report"),
        QString::null, 0, this, 0);
    viewSavedReportAction->addTo(menu);
    connect(viewSavedReportAction, SIGNAL(activated()), this, 
            SLOT(viewSavedReport()));
}

void TimeSleuthMainWindow::syncServer()
{
    document->save();

    Synchronize dlg(document, this, 0, Qt::WType_Modal);
    dlg.showMaximized();
    qApp->enter_loop();

    if (!dlg.complete)
    {
        document->load();
        QMessageBox::warning(this, 
            tr("Synchronization Incomplete"),
            tr("Synchronization was not\n"
                "successful. Data has reverted\n"
                "to pre-synchronization state."),
            QMessageBox::Ok, 0);
    }
    
    updateView();
}

void TimeSleuthMainWindow::addSettingsMenuItems(QPopupMenu *menu)
{
    // Activites
    activitiesAction = new QAction( tr("Activities"),
                                QString::null, 0, this, 0);
    activitiesAction->addTo( menu );
    connect(activitiesAction, SIGNAL(activated()), this, 
        SLOT(maintainActivities()));
          

    // Categories
    categoriesAction = new QAction( tr("Categories"),
                                QString::null, 0, this, 0);
    categoriesAction->addTo( menu );
    connect(categoriesAction, SIGNAL(activated()), this,
        SLOT(maintainCategories()));

    menu->insertSeparator();

    // Allow Concurrent Tasks
    allowConcurrentTasksAction = new QAction( tr("Allow Concurrent Tasks"),
                                QString::null, 0, this, 0, true);
    allowConcurrentTasksAction->addTo( menu );
    allowConcurrentTasksAction->setOn(
        document->configuration.allowConcurrentTasks());
    connect(allowConcurrentTasksAction, SIGNAL(toggled(bool)), this, 
        SLOT(concurrentToggled(bool)));

    // Operating Mode
    operatingModeAction = new QAction( tr("Operating Mode"),
                                QString::null, 0, this, 0);
    operatingModeAction->addTo( menu );
    connect(operatingModeAction, SIGNAL(activated()), this, 
            SLOT(operatingMode()));
}

void TimeSleuthMainWindow::viewSavedReport()
{
    FileSelectDialog dlg(this, 0);
    dlg.showMaximized();
    qApp->enter_loop();
    if (!dlg.filename.isEmpty())
    {
        DocLnk lnk(dlg.filename);
        showDocument(lnk);
    }
}

void TimeSleuthMainWindow::setDocument(const QString& applnk_filename)
{
    DocLnk lnk(applnk_filename);
    showDocument(lnk);
}

void TimeSleuthMainWindow::showDocument(const DocLnk& lnk)
{
    QString data;
    FileManager fm;
    if (!fm.loadFile(lnk, data))
        return;
    ModalWidget dlg(this, "Report", Qt::WType_Modal);
    dlg.setCaption( tr( "TimeSleuth - Report" ) );
    QTextView* tv = new QTextView(&dlg);
    tv->setGeometry(QRect(1, 1, 236, 280));
    tv->setText(data);
    tv->show();
    dlg.showMaximized();
    qApp->enter_loop();
}

void TimeSleuthMainWindow::maintainCategories()
{
    CategoryList dlg(document, this, "CategoryList", Qt::WType_Modal);
    dlg.showMaximized();
    qApp->enter_loop();
    updateView();
    showBalloonTips();
}

void TimeSleuthMainWindow::maintainActivities()
{
    ActivityList dlg(document, this, "ActivityList", Qt::WType_Modal);
    dlg.showMaximized();
    qApp->enter_loop();
    updateView();
    showBalloonTips();
}

void TimeSleuthMainWindow::operatingMode()
{
    Configuration::OperatingMode previousMode = document->configuration.mode();

    OperatingMode dlg(this, 0, true);
    dlg.showMaximized();
    dlg.txtServer->setText(document->configuration.serverAddress());
    dlg.txtPassword->setText(document->configuration.serverPassword());
    dlg.txtUserID->setText(document->configuration.serverUserID());
    if (document->configuration.mode() == 
        Configuration::PERSONAL_MODE)
    {
        dlg.radPersonal->setChecked(true);
    }
    else
    {
        dlg.radGroup->setChecked(true);
    }
    
    if (dlg.exec())
    {
        
        if (dlg.radPersonal->isChecked())
        {
            document->configuration.setMode(Configuration::PERSONAL_MODE);
        }
        else
        {
            if (previousMode != Configuration::GROUP_MODE)
            {
                if (document->tasks.count() > 0 ||
                    document->getNumActiveActivities() > 0 ||
                    document->getNumActiveCategories() > 0)
                {
                    if (QMessageBox::warning(this, tr("Change to Group Mode"),
                            tr("Changing to Group Mode will\n"
                               "result in the deletion of\n"
                               "all local Tasks, Activities\n"
                               "and Categories.\n\nProceed?"), QMessageBox::Yes,
                        QMessageBox::No|QMessageBox::Default|QMessageBox::Escape) 
                        != QMessageBox::Yes)
                    {
                        // Abort
                        return; 
                    }
                }
                document->clear();
            }

            document->configuration.setMode(Configuration::GROUP_MODE);
            document->configuration.setServerAddress(dlg.txtServer->text());
            document->configuration.setServerUserID(dlg.txtUserID->text());
            document->configuration.setServerPassword(dlg.txtPassword->text());
        }
        updateView();
    }
    
    showBalloonTips();
}

void TimeSleuthMainWindow::purge()
{
    PurgeTasks dlg(this, 0, true);
    dlg.showMaximized();
    dlg.setDateTime(getTSTime());

    if (dlg.exec())
    {
        QList<TSTask> deleted;
        deleted.setAutoDelete(true);

        QListIterator<TSTask> it(document->tasks);
        for( ; it.current(); ++it)
        {
            TSTask* t = it.current();
            if (t->stop().isValid() && (t->stop() < dlg.getDateTime()))
                deleted.append(t);
        }

        for(TSTask* t = deleted.first(); t; t = deleted.next())
            document->removeTask(t);

        view->selectedTask = 0;
        updateView();
    }
}

void TimeSleuthMainWindow::concurrentToggled(bool state)
{
    bool stopTasks = false;

    if (!state && document->activeTasks())
    {
        if (QMessageBox::warning(this, tr("Disable Concurrent Tasks"),
            tr("This will stop any active tasks.\nProceed?"), QMessageBox::Yes,
            QMessageBox::No|QMessageBox::Default|QMessageBox::Escape) 
            == QMessageBox::Yes)
        {
            stopTasks = true;
        }
        else
        {
            allowConcurrentTasksAction->blockSignals(true);
            allowConcurrentTasksAction->setOn(true);
            allowConcurrentTasksAction->blockSignals(false);
            return;
        }
    }

    if (stopTasks)
    {
        QDateTime now = getTSTime();
        bool zeroDurationMessageDisplayed = false;
        QListIterator<TSTask> it(document->tasks);
        TSTask* task = it.current();
        while(task)
        {
            if (!task->stop().isValid())
            {
                // Need to stop this task now
                if (now < task->start())
                {
                    QMessageBox::information(this, 
                     tr("Cannot Disable Concurrent Tasks"),
                     tr("An active task cannot be stopped\n"
                     "now because its start time is in\n"
                     "the future. To set a specific\n"
                     "future stop time, you can Edit\n"
                     "the task."), 
                     QMessageBox::Ok);
                    allowConcurrentTasksAction->blockSignals(true);
                    allowConcurrentTasksAction->setOn(true);
                    allowConcurrentTasksAction->blockSignals(false);
                    return;
                }
                else if (now == task->start())
                {
                    document->removeTask(task);
                    updateView();
                    delete task;
                    if (!zeroDurationMessageDisplayed)
                    {
                        zeroDurationMessageDisplayed = true;
                        QMessageBox::information(this, 
                         tr("Active Task Deleted"),
                         tr("One or more active tasks have\n"
                         "been deleted because their\n"
                         "duration was less than\n" 
                         "a minute."), 
                         QMessageBox::Ok);
                    }
                    // iterator already points to next item
                    task = it.current();
                    continue;
                }
                else
                {
                    document->stopTask(task, now);
                    updateView(task);
                }
            }
            ++it;
            task = it.current();
        }
    }

    document->configuration.setAllowConcurrentTasks(state);
}

void TimeSleuthMainWindow::setupReports()
{
    // Initialize reports
    reports.setAutoDelete(true);
    reports.append(new PeriodSummaryReport(this, document));
    reports.append(new PeriodTimeSheetReport(this, document));
}

void TimeSleuthMainWindow::setupCentralWidget()
{
    view = new MainWidget(this);
    setCentralWidget(view);
}

void TimeSleuthMainWindow::flush()
{

    // TODO implement sync
    document->save();
}

void TimeSleuthMainWindow::reload()
{
    // TODO implement sync
    document->load();
    updateView();
}

void TimeSleuthMainWindow::newTask()
{
    TaskEdit dlg(0, this, 0, TRUE);
    dlg.showMaximized();
    dlg.setCaption(tr("TimeSleuth - New Task"));

    if (dlg.exec())
    {
        // OK
        TSTask* task = new TSTask();
        task->id = document->allocateId();
        if (dlg.radStartNow->isChecked())
            task->setStart(dlg.now);
        else if (dlg.radStartSpecified->isChecked())
            task->setStart(dlg.getStartDateTime());

        if (dlg.radStopNow->isChecked())
            task->setStop(dlg.now);
        else if (dlg.radStopSpecified->isChecked())
            task->setStop(dlg.getStopDateTime());
        else if (dlg.radStopLater->isChecked())
            task->setStop(QDateTime()); // null date time	
        task->activity = dlg.activity;    
        document->tasks.append(task);
        if (!stopOtherTasks(task, dlg.now, false))
        {
            document->removeTask(task);
            delete task;
            return;
        }
        document->includeShortcut(task);
        view->selectedTask = task;
        if (task->stop().isValid())
            document->stopTask(task, task->stop());
        document->tasks.sort();
        updateView();

        showBalloonTips();
    }
}


/*
 * Returns the current local date and time
 * with seconds=0 and milliseconds=0
 */
QDateTime TimeSleuthMainWindow::getTSTime(bool withSeconds = false)
{
    QDateTime dt = QDateTime::currentDateTime();
    dt.setTime(QTime(dt.time().hour(), dt.time().minute(),
               withSeconds ? dt.time().second() : 0));
    return dt;
}

